from __future__ import annotations
from typing import Dict, Any
import dataclasses
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="PricingHeartbeat")


@dataclasses.dataclass
class PricingHeartbeat:
    """A PricingHeartbeat object is injected into the Pricing stream to ensure that the HTTP connection remains active.

    Attributes:
        type (Optional[str]): The string "HEARTBEAT"
        time (Optional[str]): The date/time when the Heartbeat was created."""

    type: Optional[str]
    time: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        type = d.pop("type", None)
        time = d.pop("time", None)
        pricing_heartbeat = cls(type=type, time=time)
        pricing_heartbeat.additional_properties = d
        return pricing_heartbeat

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Literal, Set, cast

TakeProfitOrderTransactionReason = Literal["CLIENT_ORDER", "ON_FILL", "REPLACEMENT"]
TAKE_PROFIT_ORDER_TRANSACTION_REASON_VALUES: Set[TakeProfitOrderTransactionReason] = {
    "CLIENT_ORDER",
    "ON_FILL",
    "REPLACEMENT",
}


def check_take_profit_order_transaction_reason(
    value: str,
) -> TakeProfitOrderTransactionReason:
    if value in TAKE_PROFIT_ORDER_TRANSACTION_REASON_VALUES:
        return cast(TakeProfitOrderTransactionReason, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {TAKE_PROFIT_ORDER_TRANSACTION_REASON_VALUES!r}"
    )



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .client_extensions import ClientExtensions
from .stop_loss_details import StopLossDetails
from .stop_order_position_fill import StopOrderPositionFill
from .stop_order_position_fill import check_stop_order_position_fill
from .stop_order_state import StopOrderState
from .stop_order_state import check_stop_order_state
from .stop_order_time_in_force import StopOrderTimeInForce
from .stop_order_time_in_force import check_stop_order_time_in_force
from .stop_order_trigger_condition import StopOrderTriggerCondition
from .stop_order_trigger_condition import check_stop_order_trigger_condition
from .stop_order_type import StopOrderType
from .stop_order_type import check_stop_order_type
from .take_profit_details import TakeProfitDetails
from .trailing_stop_loss_details import TrailingStopLossDetails
from typing import List, Optional, Type, TypeVar, cast

T = TypeVar("T", bound="StopOrder")


@dataclasses.dataclass
class StopOrder:
    """A StopOrder is an order that is created with a price threshold, and will only be filled by a price that is equal to
    or worse than the threshold.

        Attributes:
            id (Optional[str]): The Order's identifier, unique within the Order's Account.
            create_time (Optional[str]): The time when the Order was created.
            state (Optional[StopOrderState]): The current state of the Order.
            client_extensions (Optional[ClientExtensions]): A ClientExtensions object allows a client to attach a
                clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
                your account is associated with MT4.
            type (Optional[StopOrderType]): The type of the Order. Always set to "STOP" for Stop Orders.
            instrument (Optional[str]): The Stop Order's Instrument.
            units (Optional[str]): The quantity requested to be filled by the Stop Order. A posititive number of units
                results in a long Order, and a negative number of units results in a short Order.
            price (Optional[str]): The price threshold specified for the Stop Order. The Stop Order will only be filled
                by a market price that is equal to or worse than this price.
            price_bound (Optional[str]): The worst market price that may be used to fill this Stop Order. If the market
                gaps and crosses through both the price and the priceBound, the Stop Order will be cancelled instead of being
                filled.
            time_in_force (Optional[StopOrderTimeInForce]): The time-in-force requested for the Stop Order.
            gtd_time (Optional[str]): The date/time when the Stop Order will be cancelled if its timeInForce is "GTD".
            position_fill (Optional[StopOrderPositionFill]): Specification of how Positions in the Account are modified
                when the Order is filled.
            trigger_condition (Optional[StopOrderTriggerCondition]): Specification of which price component should be
                used when determining if an Order should be triggered and filled. This allows Orders to be triggered based on
                the bid, ask, mid, default (ask for buy, bid for sell) or inverse (ask for sell, bid for buy) price depending on
                the desired behaviour. Orders are always filled using their default price component.
                This feature is only provided through the REST API. Clients who choose to specify a non-default trigger
                condition will not see it reflected in any of OANDA's proprietary or partner trading platforms, their
                transaction history or their account statements. OANDA platforms always assume that an Order's trigger condition
                is set to the default value when indicating the distance from an Order's trigger price, and will always provide
                the default trigger condition when creating or modifying an Order.
                A special restriction applies when creating a guaranteed Stop Loss Order. In this case the TriggerCondition
                value must either be "DEFAULT", or the "natural" trigger side "DEFAULT" results in. So for a Stop Loss Order for
                a long trade valid values are "DEFAULT" and "BID", and for short trades "DEFAULT" and "ASK" are valid.
            take_profit_on_fill (Optional[TakeProfitDetails]): TakeProfitDetails specifies the details of a Take Profit
                Order to be created on behalf of a client. This may happen when an Order is filled that opens a Trade requiring
                a Take Profit, or when a Trade's dependent Take Profit Order is modified directly through the Trade.
            stop_loss_on_fill (Optional[StopLossDetails]): StopLossDetails specifies the details of a Stop Loss Order to
                be created on behalf of a client. This may happen when an Order is filled that opens a Trade requiring a Stop
                Loss, or when a Trade's dependent Stop Loss Order is modified directly through the Trade.
            trailing_stop_loss_on_fill (Optional[TrailingStopLossDetails]): TrailingStopLossDetails specifies the
                details of a Trailing Stop Loss Order to be created on behalf of a client. This may happen when an Order is
                filled that opens a Trade requiring a Trailing Stop Loss, or when a Trade's dependent Trailing Stop Loss Order
                is modified directly through the Trade.
            trade_client_extensions (Optional[ClientExtensions]): A ClientExtensions object allows a client to attach a
                clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
                your account is associated with MT4.
            filling_transaction_id (Optional[str]): ID of the Transaction that filled this Order (only provided when the
                Order's state is FILLED)
            filled_time (Optional[str]): Date/time when the Order was filled (only provided when the Order's state is
                FILLED)
            trade_opened_id (Optional[str]): Trade ID of Trade opened when the Order was filled (only provided when the
                Order's state is FILLED and a Trade was opened as a result of the fill)
            trade_reduced_id (Optional[str]): Trade ID of Trade reduced when the Order was filled (only provided when
                the Order's state is FILLED and a Trade was reduced as a result of the fill)
            trade_closed_i_ds (Optional[List[str]]): Trade IDs of Trades closed when the Order was filled (only provided
                when the Order's state is FILLED and one or more Trades were closed as a result of the fill)
            cancelling_transaction_id (Optional[str]): ID of the Transaction that cancelled the Order (only provided
                when the Order's state is CANCELLED)
            cancelled_time (Optional[str]): Date/time when the Order was cancelled (only provided when the state of the
                Order is CANCELLED)
            replaces_order_id (Optional[str]): The ID of the Order that was replaced by this Order (only provided if
                this Order was created as part of a cancel/replace).
            replaced_by_order_id (Optional[str]): The ID of the Order that replaced this Order (only provided if this
                Order was cancelled as part of a cancel/replace)."""

    id: Optional[str]
    create_time: Optional[str]
    state: Optional[StopOrderState]
    client_extensions: Optional["ClientExtensions"]
    type: Optional[StopOrderType]
    instrument: Optional[str]
    units: Optional[str]
    price: Optional[str]
    price_bound: Optional[str]
    time_in_force: Optional[StopOrderTimeInForce]
    gtd_time: Optional[str]
    position_fill: Optional[StopOrderPositionFill]
    trigger_condition: Optional[StopOrderTriggerCondition]
    take_profit_on_fill: Optional["TakeProfitDetails"]
    stop_loss_on_fill: Optional["StopLossDetails"]
    trailing_stop_loss_on_fill: Optional["TrailingStopLossDetails"]
    trade_client_extensions: Optional["ClientExtensions"]
    filling_transaction_id: Optional[str]
    filled_time: Optional[str]
    trade_opened_id: Optional[str]
    trade_reduced_id: Optional[str]
    trade_closed_i_ds: Optional[List[str]]
    cancelling_transaction_id: Optional[str]
    cancelled_time: Optional[str]
    replaces_order_id: Optional[str]
    replaced_by_order_id: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .take_profit_details import TakeProfitDetails
        from .trailing_stop_loss_details import TrailingStopLossDetails
        from .client_extensions import ClientExtensions
        from .stop_loss_details import StopLossDetails

        d = src_dict.copy()
        id = d.pop("id", None)
        create_time = d.pop("createTime", None)
        _state = d.pop("state", None)
        state: Optional[StopOrderState]
        if _state is None:
            state = None
        else:
            state = check_stop_order_state(_state)
        _client_extensions = d.pop("clientExtensions", None)
        client_extensions: Optional[ClientExtensions]
        if _client_extensions is None:
            client_extensions = None
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)
        _type = d.pop("type", None)
        type: Optional[StopOrderType]
        if _type is None:
            type = None
        else:
            type = check_stop_order_type(_type)
        instrument = d.pop("instrument", None)
        units = d.pop("units", None)
        price = d.pop("price", None)
        price_bound = d.pop("priceBound", None)
        _time_in_force = d.pop("timeInForce", None)
        time_in_force: Optional[StopOrderTimeInForce]
        if _time_in_force is None:
            time_in_force = None
        else:
            time_in_force = check_stop_order_time_in_force(_time_in_force)
        gtd_time = d.pop("gtdTime", None)
        _position_fill = d.pop("positionFill", None)
        position_fill: Optional[StopOrderPositionFill]
        if _position_fill is None:
            position_fill = None
        else:
            position_fill = check_stop_order_position_fill(_position_fill)
        _trigger_condition = d.pop("triggerCondition", None)
        trigger_condition: Optional[StopOrderTriggerCondition]
        if _trigger_condition is None:
            trigger_condition = None
        else:
            trigger_condition = check_stop_order_trigger_condition(_trigger_condition)
        _take_profit_on_fill = d.pop("takeProfitOnFill", None)
        take_profit_on_fill: Optional[TakeProfitDetails]
        if _take_profit_on_fill is None:
            take_profit_on_fill = None
        else:
            take_profit_on_fill = TakeProfitDetails.from_dict(_take_profit_on_fill)
        _stop_loss_on_fill = d.pop("stopLossOnFill", None)
        stop_loss_on_fill: Optional[StopLossDetails]
        if _stop_loss_on_fill is None:
            stop_loss_on_fill = None
        else:
            stop_loss_on_fill = StopLossDetails.from_dict(_stop_loss_on_fill)
        _trailing_stop_loss_on_fill = d.pop("trailingStopLossOnFill", None)
        trailing_stop_loss_on_fill: Optional[TrailingStopLossDetails]
        if _trailing_stop_loss_on_fill is None:
            trailing_stop_loss_on_fill = None
        else:
            trailing_stop_loss_on_fill = TrailingStopLossDetails.from_dict(
                _trailing_stop_loss_on_fill
            )
        _trade_client_extensions = d.pop("tradeClientExtensions", None)
        trade_client_extensions: Optional[ClientExtensions]
        if _trade_client_extensions is None:
            trade_client_extensions = None
        else:
            trade_client_extensions = ClientExtensions.from_dict(
                _trade_client_extensions
            )
        filling_transaction_id = d.pop("fillingTransactionID", None)
        filled_time = d.pop("filledTime", None)
        trade_opened_id = d.pop("tradeOpenedID", None)
        trade_reduced_id = d.pop("tradeReducedID", None)
        trade_closed_i_ds = cast(List[str], d.pop("tradeClosedIDs", None))
        cancelling_transaction_id = d.pop("cancellingTransactionID", None)
        cancelled_time = d.pop("cancelledTime", None)
        replaces_order_id = d.pop("replacesOrderID", None)
        replaced_by_order_id = d.pop("replacedByOrderID", None)
        stop_order = cls(
            id=id,
            create_time=create_time,
            state=state,
            client_extensions=client_extensions,
            type=type,
            instrument=instrument,
            units=units,
            price=price,
            price_bound=price_bound,
            time_in_force=time_in_force,
            gtd_time=gtd_time,
            position_fill=position_fill,
            trigger_condition=trigger_condition,
            take_profit_on_fill=take_profit_on_fill,
            stop_loss_on_fill=stop_loss_on_fill,
            trailing_stop_loss_on_fill=trailing_stop_loss_on_fill,
            trade_client_extensions=trade_client_extensions,
            filling_transaction_id=filling_transaction_id,
            filled_time=filled_time,
            trade_opened_id=trade_opened_id,
            trade_reduced_id=trade_reduced_id,
            trade_closed_i_ds=trade_closed_i_ds,
            cancelling_transaction_id=cancelling_transaction_id,
            cancelled_time=cancelled_time,
            replaces_order_id=replaces_order_id,
            replaced_by_order_id=replaced_by_order_id,
        )
        stop_order.additional_properties = d
        return stop_order

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from typing import List, Optional, Type, TypeVar, cast

T = TypeVar("T", bound="PositionSide")


@dataclasses.dataclass
class PositionSide:
    """The representation of a Position for a single direction (long or short).

    Attributes:
        units (Optional[str]): Number of units in the position (negative value indicates short position, positive
            indicates long position).
        average_price (Optional[str]): Volume-weighted average of the underlying Trade open prices for the Position.
        trade_i_ds (Optional[List[str]]): List of the open Trade IDs which contribute to the open Position.
        pl (Optional[str]): Profit/loss realized by the PositionSide over the lifetime of the Account.
        unrealized_pl (Optional[str]): The unrealized profit/loss of all open Trades that contribute to this
            PositionSide.
        resettable_pl (Optional[str]): Profit/loss realized by the PositionSide since the Account's resettablePL was
            last reset by the client.
        financing (Optional[str]): The total amount of financing paid/collected for this PositionSide over the
            lifetime of the Account.
        guaranteed_execution_fees (Optional[str]): The total amount of fees charged over the lifetime of the Account
            for the execution of guaranteed Stop Loss Orders attached to Trades for this PositionSide."""

    units: Optional[str]
    average_price: Optional[str]
    trade_i_ds: Optional[List[str]]
    pl: Optional[str]
    unrealized_pl: Optional[str]
    resettable_pl: Optional[str]
    financing: Optional[str]
    guaranteed_execution_fees: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        units = d.pop("units", None)
        average_price = d.pop("averagePrice", None)
        trade_i_ds = cast(List[str], d.pop("tradeIDs", None))
        pl = d.pop("pl", None)
        unrealized_pl = d.pop("unrealizedPL", None)
        resettable_pl = d.pop("resettablePL", None)
        financing = d.pop("financing", None)
        guaranteed_execution_fees = d.pop("guaranteedExecutionFees", None)
        position_side = cls(
            units=units,
            average_price=average_price,
            trade_i_ds=trade_i_ds,
            pl=pl,
            unrealized_pl=unrealized_pl,
            resettable_pl=resettable_pl,
            financing=financing,
            guaranteed_execution_fees=guaranteed_execution_fees,
        )
        position_side.additional_properties = d
        return position_side

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union

if TYPE_CHECKING:
    from ..models.calculated_position_state import CalculatedPositionState
    from ..models.dynamic_order_state import DynamicOrderState
    from ..models.calculated_trade_state import CalculatedTradeState


T = TypeVar("T", bound="AccountChangesState")


@_attrs_define
class AccountChangesState:
    """An AccountState Object is used to represent an Account's current price-dependent state. Price-dependent Account
    state is dependent on OANDA's current Prices, and includes things like unrealized PL, NAV and Trailing Stop Loss
    Order state.

        Attributes:
            unrealized_pl (Union[Unset, str]): The total unrealized profit/loss for all Trades currently open in the
                Account.
            nav (Union[Unset, str]): The net asset value of the Account. Equal to Account balance + unrealizedPL.
            margin_used (Union[Unset, str]): Margin currently used for the Account.
            margin_available (Union[Unset, str]): Margin available for Account currency.
            position_value (Union[Unset, str]): The value of the Account's open positions represented in the Account's home
                currency.
            margin_closeout_unrealized_pl (Union[Unset, str]): The Account's margin closeout unrealized PL.
            margin_closeout_nav (Union[Unset, str]): The Account's margin closeout NAV.
            margin_closeout_margin_used (Union[Unset, str]): The Account's margin closeout margin used.
            margin_closeout_percent (Union[Unset, str]): The Account's margin closeout percentage. When this value is 1.0 or
                above the Account is in a margin closeout situation.
            margin_closeout_position_value (Union[Unset, str]): The value of the Account's open positions as used for margin
                closeout calculations represented in the Account's home currency.
            withdrawal_limit (Union[Unset, str]): The current WithdrawalLimit for the account which will be zero or a
                positive value indicating how much can be withdrawn from the account.
            margin_call_margin_used (Union[Unset, str]): The Account's margin call margin used.
            margin_call_percent (Union[Unset, str]): The Account's margin call percentage. When this value is 1.0 or above
                the Account is in a margin call situation.
            orders (Union[Unset, List['DynamicOrderState']]): The price-dependent state of each pending Order in the
                Account.
            trades (Union[Unset, List['CalculatedTradeState']]): The price-dependent state for each open Trade in the
                Account.
            positions (Union[Unset, List['CalculatedPositionState']]): The price-dependent state for each open Position in
                the Account.
    """

    unrealized_pl: Union[Unset, str] = UNSET
    nav: Union[Unset, str] = UNSET
    margin_used: Union[Unset, str] = UNSET
    margin_available: Union[Unset, str] = UNSET
    position_value: Union[Unset, str] = UNSET
    margin_closeout_unrealized_pl: Union[Unset, str] = UNSET
    margin_closeout_nav: Union[Unset, str] = UNSET
    margin_closeout_margin_used: Union[Unset, str] = UNSET
    margin_closeout_percent: Union[Unset, str] = UNSET
    margin_closeout_position_value: Union[Unset, str] = UNSET
    withdrawal_limit: Union[Unset, str] = UNSET
    margin_call_margin_used: Union[Unset, str] = UNSET
    margin_call_percent: Union[Unset, str] = UNSET
    orders: Union[Unset, List["DynamicOrderState"]] = UNSET
    trades: Union[Unset, List["CalculatedTradeState"]] = UNSET
    positions: Union[Unset, List["CalculatedPositionState"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        unrealized_pl = self.unrealized_pl

        nav = self.nav

        margin_used = self.margin_used

        margin_available = self.margin_available

        position_value = self.position_value

        margin_closeout_unrealized_pl = self.margin_closeout_unrealized_pl

        margin_closeout_nav = self.margin_closeout_nav

        margin_closeout_margin_used = self.margin_closeout_margin_used

        margin_closeout_percent = self.margin_closeout_percent

        margin_closeout_position_value = self.margin_closeout_position_value

        withdrawal_limit = self.withdrawal_limit

        margin_call_margin_used = self.margin_call_margin_used

        margin_call_percent = self.margin_call_percent

        orders: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.orders, Unset):
            orders = []
            for orders_item_data in self.orders:
                orders_item = orders_item_data.to_dict()
                orders.append(orders_item)

        trades: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.trades, Unset):
            trades = []
            for trades_item_data in self.trades:
                trades_item = trades_item_data.to_dict()
                trades.append(trades_item)

        positions: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.positions, Unset):
            positions = []
            for positions_item_data in self.positions:
                positions_item = positions_item_data.to_dict()
                positions.append(positions_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if unrealized_pl is not UNSET:
            field_dict["unrealizedPL"] = unrealized_pl
        if nav is not UNSET:
            field_dict["NAV"] = nav
        if margin_used is not UNSET:
            field_dict["marginUsed"] = margin_used
        if margin_available is not UNSET:
            field_dict["marginAvailable"] = margin_available
        if position_value is not UNSET:
            field_dict["positionValue"] = position_value
        if margin_closeout_unrealized_pl is not UNSET:
            field_dict["marginCloseoutUnrealizedPL"] = margin_closeout_unrealized_pl
        if margin_closeout_nav is not UNSET:
            field_dict["marginCloseoutNAV"] = margin_closeout_nav
        if margin_closeout_margin_used is not UNSET:
            field_dict["marginCloseoutMarginUsed"] = margin_closeout_margin_used
        if margin_closeout_percent is not UNSET:
            field_dict["marginCloseoutPercent"] = margin_closeout_percent
        if margin_closeout_position_value is not UNSET:
            field_dict["marginCloseoutPositionValue"] = margin_closeout_position_value
        if withdrawal_limit is not UNSET:
            field_dict["withdrawalLimit"] = withdrawal_limit
        if margin_call_margin_used is not UNSET:
            field_dict["marginCallMarginUsed"] = margin_call_margin_used
        if margin_call_percent is not UNSET:
            field_dict["marginCallPercent"] = margin_call_percent
        if orders is not UNSET:
            field_dict["orders"] = orders
        if trades is not UNSET:
            field_dict["trades"] = trades
        if positions is not UNSET:
            field_dict["positions"] = positions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.calculated_position_state import CalculatedPositionState
        from ..models.dynamic_order_state import DynamicOrderState
        from ..models.calculated_trade_state import CalculatedTradeState

        d = src_dict.copy()
        unrealized_pl = d.pop("unrealizedPL", UNSET)

        nav = d.pop("NAV", UNSET)

        margin_used = d.pop("marginUsed", UNSET)

        margin_available = d.pop("marginAvailable", UNSET)

        position_value = d.pop("positionValue", UNSET)

        margin_closeout_unrealized_pl = d.pop("marginCloseoutUnrealizedPL", UNSET)

        margin_closeout_nav = d.pop("marginCloseoutNAV", UNSET)

        margin_closeout_margin_used = d.pop("marginCloseoutMarginUsed", UNSET)

        margin_closeout_percent = d.pop("marginCloseoutPercent", UNSET)

        margin_closeout_position_value = d.pop("marginCloseoutPositionValue", UNSET)

        withdrawal_limit = d.pop("withdrawalLimit", UNSET)

        margin_call_margin_used = d.pop("marginCallMarginUsed", UNSET)

        margin_call_percent = d.pop("marginCallPercent", UNSET)

        orders = []
        _orders = d.pop("orders", UNSET)
        for orders_item_data in _orders or []:
            orders_item = DynamicOrderState.from_dict(orders_item_data)

            orders.append(orders_item)

        trades = []
        _trades = d.pop("trades", UNSET)
        for trades_item_data in _trades or []:
            trades_item = CalculatedTradeState.from_dict(trades_item_data)

            trades.append(trades_item)

        positions = []
        _positions = d.pop("positions", UNSET)
        for positions_item_data in _positions or []:
            positions_item = CalculatedPositionState.from_dict(positions_item_data)

            positions.append(positions_item)

        account_changes_state = cls(
            unrealized_pl=unrealized_pl,
            nav=nav,
            margin_used=margin_used,
            margin_available=margin_available,
            position_value=position_value,
            margin_closeout_unrealized_pl=margin_closeout_unrealized_pl,
            margin_closeout_nav=margin_closeout_nav,
            margin_closeout_margin_used=margin_closeout_margin_used,
            margin_closeout_percent=margin_closeout_percent,
            margin_closeout_position_value=margin_closeout_position_value,
            withdrawal_limit=withdrawal_limit,
            margin_call_margin_used=margin_call_margin_used,
            margin_call_percent=margin_call_percent,
            orders=orders,
            trades=trades,
            positions=positions,
        )

        account_changes_state.additional_properties = d
        return account_changes_state

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union

if TYPE_CHECKING:
    from ..models.order_book_bucket import OrderBookBucket


T = TypeVar("T", bound="OrderBook")


@_attrs_define
class OrderBook:
    """The representation of an instrument's order book at a point in time

    Attributes:
        instrument (Union[Unset, str]): The order book's instrument
        time (Union[Unset, str]): The time when the order book snapshot was created.
        price (Union[Unset, str]): The price (midpoint) for the order book's instrument at the time of the order book
            snapshot
        bucket_width (Union[Unset, str]): The price width for each bucket. Each bucket covers the price range from the
            bucket's price to the bucket's price + bucketWidth.
        buckets (Union[Unset, List['OrderBookBucket']]): The partitioned order book, divided into buckets using a
            default bucket width. These buckets are only provided for price ranges which actually contain order or position
            data.
    """

    instrument: Union[Unset, str] = UNSET
    time: Union[Unset, str] = UNSET
    price: Union[Unset, str] = UNSET
    bucket_width: Union[Unset, str] = UNSET
    buckets: Union[Unset, List["OrderBookBucket"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        instrument = self.instrument

        time = self.time

        price = self.price

        bucket_width = self.bucket_width

        buckets: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.buckets, Unset):
            buckets = []
            for buckets_item_data in self.buckets:
                buckets_item = buckets_item_data.to_dict()
                buckets.append(buckets_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if instrument is not UNSET:
            field_dict["instrument"] = instrument
        if time is not UNSET:
            field_dict["time"] = time
        if price is not UNSET:
            field_dict["price"] = price
        if bucket_width is not UNSET:
            field_dict["bucketWidth"] = bucket_width
        if buckets is not UNSET:
            field_dict["buckets"] = buckets

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.order_book_bucket import OrderBookBucket

        d = src_dict.copy()
        instrument = d.pop("instrument", UNSET)

        time = d.pop("time", UNSET)

        price = d.pop("price", UNSET)

        bucket_width = d.pop("bucketWidth", UNSET)

        buckets = []
        _buckets = d.pop("buckets", UNSET)
        for buckets_item_data in _buckets or []:
            buckets_item = OrderBookBucket.from_dict(buckets_item_data)

            buckets.append(buckets_item)

        order_book = cls(
            instrument=instrument,
            time=time,
            price=price,
            bucket_width=bucket_width,
            buckets=buckets,
        )

        order_book.additional_properties = d
        return order_book

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .client_price import ClientPrice
from .order_fill_transaction_reason import OrderFillTransactionReason
from .order_fill_transaction_reason import check_order_fill_transaction_reason
from .order_fill_transaction_type import OrderFillTransactionType
from .order_fill_transaction_type import check_order_fill_transaction_type
from .trade_open import TradeOpen
from .trade_reduce import TradeReduce
from typing import List, Optional, Type, TypeVar

T = TypeVar("T", bound="OrderFillTransaction")


@dataclasses.dataclass
class OrderFillTransaction:
    """An OrderFillTransaction represents the filling of an Order in the client's Account.

    Attributes:
        id (Optional[str]): The Transaction's Identifier.
        time (Optional[str]): The date/time when the Transaction was created.
        user_id (Optional[int]): The ID of the user that initiated the creation of the Transaction.
        account_id (Optional[str]): The ID of the Account the Transaction was created for.
        batch_id (Optional[str]): The ID of the "batch" that the Transaction belongs to. Transactions in the same
            batch are applied to the Account simultaneously.
        request_id (Optional[str]): The Request ID of the request which generated the transaction.
        type (Optional[OrderFillTransactionType]): The Type of the Transaction. Always set to "ORDER_FILL" for an
            OrderFillTransaction.
        order_id (Optional[str]): The ID of the Order filled.
        client_order_id (Optional[str]): The client Order ID of the Order filled (only provided if the client has
            assigned one).
        instrument (Optional[str]): The name of the filled Order's instrument.
        units (Optional[str]): The number of units filled by the OrderFill.
        gain_quote_home_conversion_factor (Optional[str]): This is the conversion factor in effect for the Account
            at the time of the OrderFill for converting any gains realized in Instrument quote units into units of the
            Account's home currency.
        loss_quote_home_conversion_factor (Optional[str]): This is the conversion factor in effect for the Account
            at the time of the OrderFill for converting any losses realized in Instrument quote units into units of the
            Account's home currency.
        price (Optional[str]): This field is now deprecated and should no longer be used. The individual
            tradesClosed, tradeReduced and tradeOpened fields contain the exact/official price each unit was filled at.
        full_vwap (Optional[str]): The price that all of the units of the OrderFill should have been filled at, in
            the absence of guaranteed price execution. This factors in the Account's current ClientPrice, used liquidity and
            the units of the OrderFill only. If no Trades were closed with their price clamped for guaranteed stop loss
            enforcement, then this value will match the price fields of each Trade opened, closed, and reduced, and they
            will all be the exact same.
        full_price (Optional[ClientPrice]): The specification of an Account-specific Price.
        reason (Optional[OrderFillTransactionReason]): The reason that an Order was filled
        pl (Optional[str]): The profit or loss incurred when the Order was filled.
        financing (Optional[str]): The financing paid or collected when the Order was filled.
        commission (Optional[str]): The commission charged in the Account's home currency as a result of filling the
            Order. The commission is always represented as a positive quantity of the Account's home currency, however it
            reduces the balance in the Account.
        guaranteed_execution_fee (Optional[str]): The total guaranteed execution fees charged for all Trades opened,
            closed or reduced with guaranteed Stop Loss Orders.
        account_balance (Optional[str]): The Account's balance after the Order was filled.
        trade_opened (Optional[TradeOpen]): A TradeOpen object represents a Trade for an instrument that was opened
            in an Account. It is found embedded in Transactions that affect the position of an instrument in the Account,
            specifically the OrderFill Transaction.
        trades_closed (Optional[List['TradeReduce']]): The Trades that were closed when the Order was filled (only
            provided if filling the Order resulted in a closing open Trades).
        trade_reduced (Optional[TradeReduce]): A TradeReduce object represents a Trade for an instrument that was
            reduced (either partially or fully) in an Account. It is found embedded in Transactions that affect the position
            of an instrument in the account, specifically the OrderFill Transaction.
        half_spread_cost (Optional[str]): The half spread cost for the OrderFill, which is the sum of the
            halfSpreadCost values in the tradeOpened, tradesClosed and tradeReduced fields. This can be a positive or
            negative value and is represented in the home currency of the Account."""

    id: Optional[str]
    time: Optional[str]
    user_id: Optional[int]
    account_id: Optional[str]
    batch_id: Optional[str]
    request_id: Optional[str]
    type: Optional[OrderFillTransactionType]
    order_id: Optional[str]
    client_order_id: Optional[str]
    instrument: Optional[str]
    units: Optional[str]
    gain_quote_home_conversion_factor: Optional[str]
    loss_quote_home_conversion_factor: Optional[str]
    price: Optional[str]
    full_vwap: Optional[str]
    full_price: Optional["ClientPrice"]
    reason: Optional[OrderFillTransactionReason]
    pl: Optional[str]
    financing: Optional[str]
    commission: Optional[str]
    guaranteed_execution_fee: Optional[str]
    account_balance: Optional[str]
    trade_opened: Optional["TradeOpen"]
    trades_closed: Optional[List["TradeReduce"]]
    trade_reduced: Optional["TradeReduce"]
    half_spread_cost: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .trade_open import TradeOpen
        from .client_price import ClientPrice
        from .trade_reduce import TradeReduce

        d = src_dict.copy()
        id = d.pop("id", None)
        time = d.pop("time", None)
        user_id = d.pop("userID", None)
        account_id = d.pop("accountID", None)
        batch_id = d.pop("batchID", None)
        request_id = d.pop("requestID", None)
        _type = d.pop("type", None)
        type: Optional[OrderFillTransactionType]
        if _type is None:
            type = None
        else:
            type = check_order_fill_transaction_type(_type)
        order_id = d.pop("orderID", None)
        client_order_id = d.pop("clientOrderID", None)
        instrument = d.pop("instrument", None)
        units = d.pop("units", None)
        gain_quote_home_conversion_factor = d.pop("gainQuoteHomeConversionFactor", None)
        loss_quote_home_conversion_factor = d.pop("lossQuoteHomeConversionFactor", None)
        price = d.pop("price", None)
        full_vwap = d.pop("fullVWAP", None)
        _full_price = d.pop("fullPrice", None)
        full_price: Optional[ClientPrice]
        if _full_price is None:
            full_price = None
        else:
            full_price = ClientPrice.from_dict(_full_price)
        _reason = d.pop("reason", None)
        reason: Optional[OrderFillTransactionReason]
        if _reason is None:
            reason = None
        else:
            reason = check_order_fill_transaction_reason(_reason)
        pl = d.pop("pl", None)
        financing = d.pop("financing", None)
        commission = d.pop("commission", None)
        guaranteed_execution_fee = d.pop("guaranteedExecutionFee", None)
        account_balance = d.pop("accountBalance", None)
        _trade_opened = d.pop("tradeOpened", None)
        trade_opened: Optional[TradeOpen]
        if _trade_opened is None:
            trade_opened = None
        else:
            trade_opened = TradeOpen.from_dict(_trade_opened)
        trades_closed = []
        _trades_closed = d.pop("tradesClosed", None)
        for trades_closed_item_data in _trades_closed or []:
            trades_closed_item = TradeReduce.from_dict(trades_closed_item_data)
            trades_closed.append(trades_closed_item)
        _trade_reduced = d.pop("tradeReduced", None)
        trade_reduced: Optional[TradeReduce]
        if _trade_reduced is None:
            trade_reduced = None
        else:
            trade_reduced = TradeReduce.from_dict(_trade_reduced)
        half_spread_cost = d.pop("halfSpreadCost", None)
        order_fill_transaction = cls(
            id=id,
            time=time,
            user_id=user_id,
            account_id=account_id,
            batch_id=batch_id,
            request_id=request_id,
            type=type,
            order_id=order_id,
            client_order_id=client_order_id,
            instrument=instrument,
            units=units,
            gain_quote_home_conversion_factor=gain_quote_home_conversion_factor,
            loss_quote_home_conversion_factor=loss_quote_home_conversion_factor,
            price=price,
            full_vwap=full_vwap,
            full_price=full_price,
            reason=reason,
            pl=pl,
            financing=financing,
            commission=commission,
            guaranteed_execution_fee=guaranteed_execution_fee,
            account_balance=account_balance,
            trade_opened=trade_opened,
            trades_closed=trades_closed,
            trade_reduced=trade_reduced,
            half_spread_cost=half_spread_cost,
        )
        order_fill_transaction.additional_properties = d
        return order_fill_transaction

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .client_price_status import ClientPriceStatus
from .client_price_status import check_client_price_status
from .price_bucket import PriceBucket
from .quote_home_conversion_factors import QuoteHomeConversionFactors
from .units_available import UnitsAvailable
from typing import List, Optional, Type, TypeVar

T = TypeVar("T", bound="ClientPrice")


@dataclasses.dataclass
class ClientPrice:
    """The specification of an Account-specific Price.

    Attributes:
        type (Optional[str]): The string "PRICE". Used to identify the a Price object when found in a stream.
        instrument (Optional[str]): The Price's Instrument.
        time (Optional[str]): The date/time when the Price was created
        status (Optional[ClientPriceStatus]): The status of the Price.
        tradeable (Optional[bool]): Flag indicating if the Price is tradeable or not
        bids (Optional[List['PriceBucket']]): The list of prices and liquidity available on the Instrument's bid
            side. It is possible for this list to be empty if there is no bid liquidity currently available for the
            Instrument in the Account.
        asks (Optional[List['PriceBucket']]): The list of prices and liquidity available on the Instrument's ask
            side. It is possible for this list to be empty if there is no ask liquidity currently available for the
            Instrument in the Account.
        closeout_bid (Optional[str]): The closeout bid Price. This Price is used when a bid is required to closeout
            a Position (margin closeout or manual) yet there is no bid liquidity. The closeout bid is never used to open a
            new position.
        closeout_ask (Optional[str]): The closeout ask Price. This Price is used when a ask is required to closeout
            a Position (margin closeout or manual) yet there is no ask liquidity. The closeout ask is never used to open a
            new position.
        quote_home_conversion_factors (Optional[QuoteHomeConversionFactors]): QuoteHomeConversionFactors represents
            the factors that can be used used to convert quantities of a Price's Instrument's quote currency into the
            Account's home currency.
        units_available (Optional[UnitsAvailable]): Representation of how many units of an Instrument are available
            to be traded by an Order depending on its postionFill option."""

    type: Optional[str]
    instrument: Optional[str]
    time: Optional[str]
    status: Optional[ClientPriceStatus]
    tradeable: Optional[bool]
    bids: Optional[List["PriceBucket"]]
    asks: Optional[List["PriceBucket"]]
    closeout_bid: Optional[str]
    closeout_ask: Optional[str]
    quote_home_conversion_factors: Optional["QuoteHomeConversionFactors"]
    units_available: Optional["UnitsAvailable"]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .quote_home_conversion_factors import QuoteHomeConversionFactors
        from .units_available import UnitsAvailable
        from .price_bucket import PriceBucket

        d = src_dict.copy()
        type = d.pop("type", None)
        instrument = d.pop("instrument", None)
        time = d.pop("time", None)
        _status = d.pop("status", None)
        status: Optional[ClientPriceStatus]
        if _status is None:
            status = None
        else:
            status = check_client_price_status(_status)
        tradeable = d.pop("tradeable", None)
        bids = []
        _bids = d.pop("bids", None)
        for bids_item_data in _bids or []:
            bids_item = PriceBucket.from_dict(bids_item_data)
            bids.append(bids_item)
        asks = []
        _asks = d.pop("asks", None)
        for asks_item_data in _asks or []:
            asks_item = PriceBucket.from_dict(asks_item_data)
            asks.append(asks_item)
        closeout_bid = d.pop("closeoutBid", None)
        closeout_ask = d.pop("closeoutAsk", None)
        _quote_home_conversion_factors = d.pop("quoteHomeConversionFactors", None)
        quote_home_conversion_factors: Optional[QuoteHomeConversionFactors]
        if _quote_home_conversion_factors is None:
            quote_home_conversion_factors = None
        else:
            quote_home_conversion_factors = QuoteHomeConversionFactors.from_dict(
                _quote_home_conversion_factors
            )
        _units_available = d.pop("unitsAvailable", None)
        units_available: Optional[UnitsAvailable]
        if _units_available is None:
            units_available = None
        else:
            units_available = UnitsAvailable.from_dict(_units_available)
        client_price = cls(
            type=type,
            instrument=instrument,
            time=time,
            status=status,
            tradeable=tradeable,
            bids=bids,
            asks=asks,
            closeout_bid=closeout_bid,
            closeout_ask=closeout_ask,
            quote_home_conversion_factors=quote_home_conversion_factors,
            units_available=units_available,
        )
        client_price.additional_properties = d
        return client_price

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .client_extensions import ClientExtensions
from .market_order_request_position_fill import MarketOrderRequestPositionFill
from .market_order_request_position_fill import check_market_order_request_position_fill
from .market_order_request_time_in_force import MarketOrderRequestTimeInForce
from .market_order_request_time_in_force import check_market_order_request_time_in_force
from .market_order_request_type import MarketOrderRequestType
from .market_order_request_type import check_market_order_request_type
from .stop_loss_details import StopLossDetails
from .take_profit_details import TakeProfitDetails
from .trailing_stop_loss_details import TrailingStopLossDetails
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="MarketOrderRequest")


@dataclasses.dataclass
class MarketOrderRequest:
    """A MarketOrderRequest specifies the parameters that may be set when creating a Market Order.

    Attributes:
        type (Optional[MarketOrderRequestType]): The type of the Order to Create. Must be set to "MARKET" when
            creating a Market Order.
        instrument (Optional[str]): The Market Order's Instrument.
        units (Optional[str]): The quantity requested to be filled by the Market Order. A posititive number of units
            results in a long Order, and a negative number of units results in a short Order.
        time_in_force (Optional[MarketOrderRequestTimeInForce]): The time-in-force requested for the Market Order.
            Restricted to FOK or IOC for a MarketOrder.
        price_bound (Optional[str]): The worst price that the client is willing to have the Market Order filled at.
        position_fill (Optional[MarketOrderRequestPositionFill]): Specification of how Positions in the Account are
            modified when the Order is filled.
        client_extensions (Optional[ClientExtensions]): A ClientExtensions object allows a client to attach a
            clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
            your account is associated with MT4.
        take_profit_on_fill (Optional[TakeProfitDetails]): TakeProfitDetails specifies the details of a Take Profit
            Order to be created on behalf of a client. This may happen when an Order is filled that opens a Trade requiring
            a Take Profit, or when a Trade's dependent Take Profit Order is modified directly through the Trade.
        stop_loss_on_fill (Optional[StopLossDetails]): StopLossDetails specifies the details of a Stop Loss Order to
            be created on behalf of a client. This may happen when an Order is filled that opens a Trade requiring a Stop
            Loss, or when a Trade's dependent Stop Loss Order is modified directly through the Trade.
        trailing_stop_loss_on_fill (Optional[TrailingStopLossDetails]): TrailingStopLossDetails specifies the
            details of a Trailing Stop Loss Order to be created on behalf of a client. This may happen when an Order is
            filled that opens a Trade requiring a Trailing Stop Loss, or when a Trade's dependent Trailing Stop Loss Order
            is modified directly through the Trade.
        trade_client_extensions (Optional[ClientExtensions]): A ClientExtensions object allows a client to attach a
            clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
            your account is associated with MT4."""

    type: Optional[MarketOrderRequestType]
    instrument: Optional[str]
    units: Optional[str]
    time_in_force: Optional[MarketOrderRequestTimeInForce]
    price_bound: Optional[str]
    position_fill: Optional[MarketOrderRequestPositionFill]
    client_extensions: Optional["ClientExtensions"]
    take_profit_on_fill: Optional["TakeProfitDetails"]
    stop_loss_on_fill: Optional["StopLossDetails"]
    trailing_stop_loss_on_fill: Optional["TrailingStopLossDetails"]
    trade_client_extensions: Optional["ClientExtensions"]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .take_profit_details import TakeProfitDetails
        from .trailing_stop_loss_details import TrailingStopLossDetails
        from .client_extensions import ClientExtensions
        from .stop_loss_details import StopLossDetails

        d = src_dict.copy()
        _type = d.pop("type", None)
        type: Optional[MarketOrderRequestType]
        if _type is None:
            type = None
        else:
            type = check_market_order_request_type(_type)
        instrument = d.pop("instrument", None)
        units = d.pop("units", None)
        _time_in_force = d.pop("timeInForce", None)
        time_in_force: Optional[MarketOrderRequestTimeInForce]
        if _time_in_force is None:
            time_in_force = None
        else:
            time_in_force = check_market_order_request_time_in_force(_time_in_force)
        price_bound = d.pop("priceBound", None)
        _position_fill = d.pop("positionFill", None)
        position_fill: Optional[MarketOrderRequestPositionFill]
        if _position_fill is None:
            position_fill = None
        else:
            position_fill = check_market_order_request_position_fill(_position_fill)
        _client_extensions = d.pop("clientExtensions", None)
        client_extensions: Optional[ClientExtensions]
        if _client_extensions is None:
            client_extensions = None
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)
        _take_profit_on_fill = d.pop("takeProfitOnFill", None)
        take_profit_on_fill: Optional[TakeProfitDetails]
        if _take_profit_on_fill is None:
            take_profit_on_fill = None
        else:
            take_profit_on_fill = TakeProfitDetails.from_dict(_take_profit_on_fill)
        _stop_loss_on_fill = d.pop("stopLossOnFill", None)
        stop_loss_on_fill: Optional[StopLossDetails]
        if _stop_loss_on_fill is None:
            stop_loss_on_fill = None
        else:
            stop_loss_on_fill = StopLossDetails.from_dict(_stop_loss_on_fill)
        _trailing_stop_loss_on_fill = d.pop("trailingStopLossOnFill", None)
        trailing_stop_loss_on_fill: Optional[TrailingStopLossDetails]
        if _trailing_stop_loss_on_fill is None:
            trailing_stop_loss_on_fill = None
        else:
            trailing_stop_loss_on_fill = TrailingStopLossDetails.from_dict(
                _trailing_stop_loss_on_fill
            )
        _trade_client_extensions = d.pop("tradeClientExtensions", None)
        trade_client_extensions: Optional[ClientExtensions]
        if _trade_client_extensions is None:
            trade_client_extensions = None
        else:
            trade_client_extensions = ClientExtensions.from_dict(
                _trade_client_extensions
            )
        market_order_request = cls(
            type=type,
            instrument=instrument,
            units=units,
            time_in_force=time_in_force,
            price_bound=price_bound,
            position_fill=position_fill,
            client_extensions=client_extensions,
            take_profit_on_fill=take_profit_on_fill,
            stop_loss_on_fill=stop_loss_on_fill,
            trailing_stop_loss_on_fill=trailing_stop_loss_on_fill,
            trade_client_extensions=trade_client_extensions,
        )
        market_order_request.additional_properties = d
        return market_order_request

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Literal, Set, cast

TrailingStopLossOrderRejectTransactionReason = Literal[
    "CLIENT_ORDER", "ON_FILL", "REPLACEMENT"
]
TRAILING_STOP_LOSS_ORDER_REJECT_TRANSACTION_REASON_VALUES: Set[
    TrailingStopLossOrderRejectTransactionReason
] = {"CLIENT_ORDER", "ON_FILL", "REPLACEMENT"}


def check_trailing_stop_loss_order_reject_transaction_reason(
    value: str,
) -> TrailingStopLossOrderRejectTransactionReason:
    if value in TRAILING_STOP_LOSS_ORDER_REJECT_TRANSACTION_REASON_VALUES:
        return cast(TrailingStopLossOrderRejectTransactionReason, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {TRAILING_STOP_LOSS_ORDER_REJECT_TRANSACTION_REASON_VALUES!r}"
    )



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union

if TYPE_CHECKING:
    from ..models.position_side import PositionSide


T = TypeVar("T", bound="Position")


@_attrs_define
class Position:
    """The specification of a Position within an Account.

    Attributes:
        instrument (Union[Unset, str]): The Position's Instrument.
        pl (Union[Unset, str]): Profit/loss realized by the Position over the lifetime of the Account.
        unrealized_pl (Union[Unset, str]): The unrealized profit/loss of all open Trades that contribute to this
            Position.
        margin_used (Union[Unset, str]): Margin currently used by the Position.
        resettable_pl (Union[Unset, str]): Profit/loss realized by the Position since the Account's resettablePL was
            last reset by the client.
        financing (Union[Unset, str]): The total amount of financing paid/collected for this instrument over the
            lifetime of the Account.
        commission (Union[Unset, str]): The total amount of commission paid for this instrument over the lifetime of the
            Account.
        guaranteed_execution_fees (Union[Unset, str]): The total amount of fees charged over the lifetime of the Account
            for the execution of guaranteed Stop Loss Orders for this instrument.
        long (Union[Unset, PositionSide]): The representation of a Position for a single direction (long or short).
        short (Union[Unset, PositionSide]): The representation of a Position for a single direction (long or short).
    """

    instrument: Union[Unset, str] = UNSET
    pl: Union[Unset, str] = UNSET
    unrealized_pl: Union[Unset, str] = UNSET
    margin_used: Union[Unset, str] = UNSET
    resettable_pl: Union[Unset, str] = UNSET
    financing: Union[Unset, str] = UNSET
    commission: Union[Unset, str] = UNSET
    guaranteed_execution_fees: Union[Unset, str] = UNSET
    long: Union[Unset, "PositionSide"] = UNSET
    short: Union[Unset, "PositionSide"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        instrument = self.instrument

        pl = self.pl

        unrealized_pl = self.unrealized_pl

        margin_used = self.margin_used

        resettable_pl = self.resettable_pl

        financing = self.financing

        commission = self.commission

        guaranteed_execution_fees = self.guaranteed_execution_fees

        long: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.long, Unset):
            long = self.long.to_dict()

        short: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.short, Unset):
            short = self.short.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if instrument is not UNSET:
            field_dict["instrument"] = instrument
        if pl is not UNSET:
            field_dict["pl"] = pl
        if unrealized_pl is not UNSET:
            field_dict["unrealizedPL"] = unrealized_pl
        if margin_used is not UNSET:
            field_dict["marginUsed"] = margin_used
        if resettable_pl is not UNSET:
            field_dict["resettablePL"] = resettable_pl
        if financing is not UNSET:
            field_dict["financing"] = financing
        if commission is not UNSET:
            field_dict["commission"] = commission
        if guaranteed_execution_fees is not UNSET:
            field_dict["guaranteedExecutionFees"] = guaranteed_execution_fees
        if long is not UNSET:
            field_dict["long"] = long
        if short is not UNSET:
            field_dict["short"] = short

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.position_side import PositionSide

        d = src_dict.copy()
        instrument = d.pop("instrument", UNSET)

        pl = d.pop("pl", UNSET)

        unrealized_pl = d.pop("unrealizedPL", UNSET)

        margin_used = d.pop("marginUsed", UNSET)

        resettable_pl = d.pop("resettablePL", UNSET)

        financing = d.pop("financing", UNSET)

        commission = d.pop("commission", UNSET)

        guaranteed_execution_fees = d.pop("guaranteedExecutionFees", UNSET)

        _long = d.pop("long", UNSET)
        long: Union[Unset, PositionSide]
        if isinstance(_long, Unset):
            long = UNSET
        else:
            long = PositionSide.from_dict(_long)

        _short = d.pop("short", UNSET)
        short: Union[Unset, PositionSide]
        if isinstance(_short, Unset):
            short = UNSET
        else:
            short = PositionSide.from_dict(_short)

        position = cls(
            instrument=instrument,
            pl=pl,
            unrealized_pl=unrealized_pl,
            margin_used=margin_used,
            resettable_pl=resettable_pl,
            financing=financing,
            commission=commission,
            guaranteed_execution_fees=guaranteed_execution_fees,
            long=long,
            short=short,
        )

        position.additional_properties = d
        return position

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations
from typing import Literal, Set, cast

StopLossOrderReason = Literal["CLIENT_ORDER", "ON_FILL", "REPLACEMENT"]
STOP_LOSS_ORDER_REASON_VALUES: Set[StopLossOrderReason] = {
    "CLIENT_ORDER",
    "ON_FILL",
    "REPLACEMENT",
}


def check_stop_loss_order_reason(value: str) -> StopLossOrderReason:
    if value in STOP_LOSS_ORDER_REASON_VALUES:
        return cast(StopLossOrderReason, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {STOP_LOSS_ORDER_REASON_VALUES!r}"
    )



from __future__ import annotations
from typing import Literal, Set, cast

OrderStateFilter = Literal["ALL", "CANCELLED", "FILLED", "PENDING", "TRIGGERED"]
ORDER_STATE_FILTER_VALUES: Set[OrderStateFilter] = {
    "ALL",
    "CANCELLED",
    "FILLED",
    "PENDING",
    "TRIGGERED",
}


def check_order_state_filter(value: str) -> OrderStateFilter:
    if value in ORDER_STATE_FILTER_VALUES:
        return cast(OrderStateFilter, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {ORDER_STATE_FILTER_VALUES!r}"
    )



from __future__ import annotations
from typing import Literal, Set, cast

PositionAggregationMode = Literal["ABSOLUTE_SUM", "MAXIMAL_SIDE", "NET_SUM"]
POSITION_AGGREGATION_MODE_VALUES: Set[PositionAggregationMode] = {
    "ABSOLUTE_SUM",
    "MAXIMAL_SIDE",
    "NET_SUM",
}


def check_position_aggregation_mode(value: str) -> PositionAggregationMode:
    if value in POSITION_AGGREGATION_MODE_VALUES:
        return cast(PositionAggregationMode, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {POSITION_AGGREGATION_MODE_VALUES!r}"
    )



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .account_summary_guaranteed_stop_loss_order_mode import (
    AccountSummaryGuaranteedStopLossOrderMode,
)
from .account_summary_guaranteed_stop_loss_order_mode import (
    check_account_summary_guaranteed_stop_loss_order_mode,
)
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="AccountSummary")


@dataclasses.dataclass
class AccountSummary:
    """A summary representation of a client's Account. The AccountSummary does not provide to full specification of pending
    Orders, open Trades and Positions.

        Attributes:
            id (Optional[str]): The Account's identifier
            alias (Optional[str]): Client-assigned alias for the Account. Only provided if the Account has an alias set
            currency (Optional[str]): The home currency of the Account
            balance (Optional[str]): The current balance of the Account.
            created_by_user_id (Optional[int]): ID of the user that created the Account.
            created_time (Optional[str]): The date/time when the Account was created.
            guaranteed_stop_loss_order_mode (Optional[AccountSummaryGuaranteedStopLossOrderMode]): The current
                guaranteed Stop Loss Order mode of the Account.
            pl (Optional[str]): The total profit/loss realized over the lifetime of the Account.
            resettable_pl (Optional[str]): The total realized profit/loss for the Account since it was last reset by the
                client.
            resettable_pl_time (Optional[str]): The date/time that the Account's resettablePL was last reset.
            financing (Optional[str]): The total amount of financing paid/collected over the lifetime of the Account.
            commission (Optional[str]): The total amount of commission paid over the lifetime of the Account.
            guaranteed_execution_fees (Optional[str]): The total amount of fees charged over the lifetime of the Account
                for the execution of guaranteed Stop Loss Orders.
            margin_rate (Optional[str]): Client-provided margin rate override for the Account. The effective margin rate
                of the Account is the lesser of this value and the OANDA margin rate for the Account's division. This value is
                only provided if a margin rate override exists for the Account.
            margin_call_enter_time (Optional[str]): The date/time when the Account entered a margin call state. Only
                provided if the Account is in a margin call.
            margin_call_extension_count (Optional[int]): The number of times that the Account's current margin call was
                extended.
            last_margin_call_extension_time (Optional[str]): The date/time of the Account's last margin call extension.
            open_trade_count (Optional[int]): The number of Trades currently open in the Account.
            open_position_count (Optional[int]): The number of Positions currently open in the Account.
            pending_order_count (Optional[int]): The number of Orders currently pending in the Account.
            hedging_enabled (Optional[bool]): Flag indicating that the Account has hedging enabled.
            last_order_fill_timestamp (Optional[str]): The date/time of the last order that was filled for this account.
            unrealized_pl (Optional[str]): The total unrealized profit/loss for all Trades currently open in the
                Account.
            nav (Optional[str]): The net asset value of the Account. Equal to Account balance + unrealizedPL.
            margin_used (Optional[str]): Margin currently used for the Account.
            margin_available (Optional[str]): Margin available for Account currency.
            position_value (Optional[str]): The value of the Account's open positions represented in the Account's home
                currency.
            margin_closeout_unrealized_pl (Optional[str]): The Account's margin closeout unrealized PL.
            margin_closeout_nav (Optional[str]): The Account's margin closeout NAV.
            margin_closeout_margin_used (Optional[str]): The Account's margin closeout margin used.
            margin_closeout_percent (Optional[str]): The Account's margin closeout percentage. When this value is 1.0 or
                above the Account is in a margin closeout situation.
            margin_closeout_position_value (Optional[str]): The value of the Account's open positions as used for margin
                closeout calculations represented in the Account's home currency.
            withdrawal_limit (Optional[str]): The current WithdrawalLimit for the account which will be zero or a
                positive value indicating how much can be withdrawn from the account.
            margin_call_margin_used (Optional[str]): The Account's margin call margin used.
            margin_call_percent (Optional[str]): The Account's margin call percentage. When this value is 1.0 or above
                the Account is in a margin call situation.
            last_transaction_id (Optional[str]): The ID of the last Transaction created for the Account."""

    id: Optional[str]
    alias: Optional[str]
    currency: Optional[str]
    balance: Optional[str]
    created_by_user_id: Optional[int]
    created_time: Optional[str]
    guaranteed_stop_loss_order_mode: Optional[AccountSummaryGuaranteedStopLossOrderMode]
    pl: Optional[str]
    resettable_pl: Optional[str]
    resettable_pl_time: Optional[str]
    financing: Optional[str]
    commission: Optional[str]
    guaranteed_execution_fees: Optional[str]
    margin_rate: Optional[str]
    margin_call_enter_time: Optional[str]
    margin_call_extension_count: Optional[int]
    last_margin_call_extension_time: Optional[str]
    open_trade_count: Optional[int]
    open_position_count: Optional[int]
    pending_order_count: Optional[int]
    hedging_enabled: Optional[bool]
    last_order_fill_timestamp: Optional[str]
    unrealized_pl: Optional[str]
    nav: Optional[str]
    margin_used: Optional[str]
    margin_available: Optional[str]
    position_value: Optional[str]
    margin_closeout_unrealized_pl: Optional[str]
    margin_closeout_nav: Optional[str]
    margin_closeout_margin_used: Optional[str]
    margin_closeout_percent: Optional[str]
    margin_closeout_position_value: Optional[str]
    withdrawal_limit: Optional[str]
    margin_call_margin_used: Optional[str]
    margin_call_percent: Optional[str]
    last_transaction_id: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", None)
        alias = d.pop("alias", None)
        currency = d.pop("currency", None)
        balance = d.pop("balance", None)
        created_by_user_id = d.pop("createdByUserID", None)
        created_time = d.pop("createdTime", None)
        _guaranteed_stop_loss_order_mode = d.pop("guaranteedStopLossOrderMode", None)
        guaranteed_stop_loss_order_mode: Optional[
            AccountSummaryGuaranteedStopLossOrderMode
        ]
        if _guaranteed_stop_loss_order_mode is None:
            guaranteed_stop_loss_order_mode = None
        else:
            guaranteed_stop_loss_order_mode = (
                check_account_summary_guaranteed_stop_loss_order_mode(
                    _guaranteed_stop_loss_order_mode
                )
            )
        pl = d.pop("pl", None)
        resettable_pl = d.pop("resettablePL", None)
        resettable_pl_time = d.pop("resettablePLTime", None)
        financing = d.pop("financing", None)
        commission = d.pop("commission", None)
        guaranteed_execution_fees = d.pop("guaranteedExecutionFees", None)
        margin_rate = d.pop("marginRate", None)
        margin_call_enter_time = d.pop("marginCallEnterTime", None)
        margin_call_extension_count = d.pop("marginCallExtensionCount", None)
        last_margin_call_extension_time = d.pop("lastMarginCallExtensionTime", None)
        open_trade_count = d.pop("openTradeCount", None)
        open_position_count = d.pop("openPositionCount", None)
        pending_order_count = d.pop("pendingOrderCount", None)
        hedging_enabled = d.pop("hedgingEnabled", None)
        last_order_fill_timestamp = d.pop("lastOrderFillTimestamp", None)
        unrealized_pl = d.pop("unrealizedPL", None)
        nav = d.pop("NAV", None)
        margin_used = d.pop("marginUsed", None)
        margin_available = d.pop("marginAvailable", None)
        position_value = d.pop("positionValue", None)
        margin_closeout_unrealized_pl = d.pop("marginCloseoutUnrealizedPL", None)
        margin_closeout_nav = d.pop("marginCloseoutNAV", None)
        margin_closeout_margin_used = d.pop("marginCloseoutMarginUsed", None)
        margin_closeout_percent = d.pop("marginCloseoutPercent", None)
        margin_closeout_position_value = d.pop("marginCloseoutPositionValue", None)
        withdrawal_limit = d.pop("withdrawalLimit", None)
        margin_call_margin_used = d.pop("marginCallMarginUsed", None)
        margin_call_percent = d.pop("marginCallPercent", None)
        last_transaction_id = d.pop("lastTransactionID", None)
        account_summary = cls(
            id=id,
            alias=alias,
            currency=currency,
            balance=balance,
            created_by_user_id=created_by_user_id,
            created_time=created_time,
            guaranteed_stop_loss_order_mode=guaranteed_stop_loss_order_mode,
            pl=pl,
            resettable_pl=resettable_pl,
            resettable_pl_time=resettable_pl_time,
            financing=financing,
            commission=commission,
            guaranteed_execution_fees=guaranteed_execution_fees,
            margin_rate=margin_rate,
            margin_call_enter_time=margin_call_enter_time,
            margin_call_extension_count=margin_call_extension_count,
            last_margin_call_extension_time=last_margin_call_extension_time,
            open_trade_count=open_trade_count,
            open_position_count=open_position_count,
            pending_order_count=pending_order_count,
            hedging_enabled=hedging_enabled,
            last_order_fill_timestamp=last_order_fill_timestamp,
            unrealized_pl=unrealized_pl,
            nav=nav,
            margin_used=margin_used,
            margin_available=margin_available,
            position_value=position_value,
            margin_closeout_unrealized_pl=margin_closeout_unrealized_pl,
            margin_closeout_nav=margin_closeout_nav,
            margin_closeout_margin_used=margin_closeout_margin_used,
            margin_closeout_percent=margin_closeout_percent,
            margin_closeout_position_value=margin_closeout_position_value,
            withdrawal_limit=withdrawal_limit,
            margin_call_margin_used=margin_call_margin_used,
            margin_call_percent=margin_call_percent,
            last_transaction_id=last_transaction_id,
        )
        account_summary.additional_properties = d
        return account_summary

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .client_extensions import ClientExtensions
from .market_if_touched_order_reject_transaction_position_fill import (
    MarketIfTouchedOrderRejectTransactionPositionFill,
)
from .market_if_touched_order_reject_transaction_position_fill import (
    check_market_if_touched_order_reject_transaction_position_fill,
)
from .market_if_touched_order_reject_transaction_reason import (
    MarketIfTouchedOrderRejectTransactionReason,
)
from .market_if_touched_order_reject_transaction_reason import (
    check_market_if_touched_order_reject_transaction_reason,
)
from .market_if_touched_order_reject_transaction_reject_reason import (
    MarketIfTouchedOrderRejectTransactionRejectReason,
)
from .market_if_touched_order_reject_transaction_reject_reason import (
    check_market_if_touched_order_reject_transaction_reject_reason,
)
from .market_if_touched_order_reject_transaction_time_in_force import (
    MarketIfTouchedOrderRejectTransactionTimeInForce,
)
from .market_if_touched_order_reject_transaction_time_in_force import (
    check_market_if_touched_order_reject_transaction_time_in_force,
)
from .market_if_touched_order_reject_transaction_trigger_condition import (
    MarketIfTouchedOrderRejectTransactionTriggerCondition,
)
from .market_if_touched_order_reject_transaction_trigger_condition import (
    check_market_if_touched_order_reject_transaction_trigger_condition,
)
from .market_if_touched_order_reject_transaction_type import (
    MarketIfTouchedOrderRejectTransactionType,
)
from .market_if_touched_order_reject_transaction_type import (
    check_market_if_touched_order_reject_transaction_type,
)
from .stop_loss_details import StopLossDetails
from .take_profit_details import TakeProfitDetails
from .trailing_stop_loss_details import TrailingStopLossDetails
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="MarketIfTouchedOrderRejectTransaction")


@dataclasses.dataclass
class MarketIfTouchedOrderRejectTransaction:
    """A MarketIfTouchedOrderRejectTransaction represents the rejection of the creation of a MarketIfTouched Order.

    Attributes:
        id (Optional[str]): The Transaction's Identifier.
        time (Optional[str]): The date/time when the Transaction was created.
        user_id (Optional[int]): The ID of the user that initiated the creation of the Transaction.
        account_id (Optional[str]): The ID of the Account the Transaction was created for.
        batch_id (Optional[str]): The ID of the "batch" that the Transaction belongs to. Transactions in the same
            batch are applied to the Account simultaneously.
        request_id (Optional[str]): The Request ID of the request which generated the transaction.
        type (Optional[MarketIfTouchedOrderRejectTransactionType]): The Type of the Transaction. Always set to
            "MARKET_IF_TOUCHED_ORDER_REJECT" in a MarketIfTouchedOrderRejectTransaction.
        instrument (Optional[str]): The MarketIfTouched Order's Instrument.
        units (Optional[str]): The quantity requested to be filled by the MarketIfTouched Order. A posititive number
            of units results in a long Order, and a negative number of units results in a short Order.
        price (Optional[str]): The price threshold specified for the MarketIfTouched Order. The MarketIfTouched
            Order will only be filled by a market price that crosses this price from the direction of the market price at
            the time when the Order was created (the initialMarketPrice). Depending on the value of the Order's price and
            initialMarketPrice, the MarketIfTouchedOrder will behave like a Limit or a Stop Order.
        price_bound (Optional[str]): The worst market price that may be used to fill this MarketIfTouched Order.
        time_in_force (Optional[MarketIfTouchedOrderRejectTransactionTimeInForce]): The time-in-force requested for
            the MarketIfTouched Order. Restricted to "GTC", "GFD" and "GTD" for MarketIfTouched Orders.
        gtd_time (Optional[str]): The date/time when the MarketIfTouched Order will be cancelled if its timeInForce
            is "GTD".
        position_fill (Optional[MarketIfTouchedOrderRejectTransactionPositionFill]): Specification of how Positions
            in the Account are modified when the Order is filled.
        trigger_condition (Optional[MarketIfTouchedOrderRejectTransactionTriggerCondition]): Specification of which
            price component should be used when determining if an Order should be triggered and filled. This allows Orders
            to be triggered based on the bid, ask, mid, default (ask for buy, bid for sell) or inverse (ask for sell, bid
            for buy) price depending on the desired behaviour. Orders are always filled using their default price component.
            This feature is only provided through the REST API. Clients who choose to specify a non-default trigger
            condition will not see it reflected in any of OANDA's proprietary or partner trading platforms, their
            transaction history or their account statements. OANDA platforms always assume that an Order's trigger condition
            is set to the default value when indicating the distance from an Order's trigger price, and will always provide
            the default trigger condition when creating or modifying an Order.
            A special restriction applies when creating a guaranteed Stop Loss Order. In this case the TriggerCondition
            value must either be "DEFAULT", or the "natural" trigger side "DEFAULT" results in. So for a Stop Loss Order for
            a long trade valid values are "DEFAULT" and "BID", and for short trades "DEFAULT" and "ASK" are valid.
        reason (Optional[MarketIfTouchedOrderRejectTransactionReason]): The reason that the Market-if-touched Order
            was initiated
        client_extensions (Optional[ClientExtensions]): A ClientExtensions object allows a client to attach a
            clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
            your account is associated with MT4.
        take_profit_on_fill (Optional[TakeProfitDetails]): TakeProfitDetails specifies the details of a Take Profit
            Order to be created on behalf of a client. This may happen when an Order is filled that opens a Trade requiring
            a Take Profit, or when a Trade's dependent Take Profit Order is modified directly through the Trade.
        stop_loss_on_fill (Optional[StopLossDetails]): StopLossDetails specifies the details of a Stop Loss Order to
            be created on behalf of a client. This may happen when an Order is filled that opens a Trade requiring a Stop
            Loss, or when a Trade's dependent Stop Loss Order is modified directly through the Trade.
        trailing_stop_loss_on_fill (Optional[TrailingStopLossDetails]): TrailingStopLossDetails specifies the
            details of a Trailing Stop Loss Order to be created on behalf of a client. This may happen when an Order is
            filled that opens a Trade requiring a Trailing Stop Loss, or when a Trade's dependent Trailing Stop Loss Order
            is modified directly through the Trade.
        trade_client_extensions (Optional[ClientExtensions]): A ClientExtensions object allows a client to attach a
            clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
            your account is associated with MT4.
        intended_replaces_order_id (Optional[str]): The ID of the Order that this Order was intended to replace
            (only provided if this Order was intended to replace an existing Order).
        reject_reason (Optional[MarketIfTouchedOrderRejectTransactionRejectReason]): The reason that the Reject
            Transaction was created"""

    id: Optional[str]
    time: Optional[str]
    user_id: Optional[int]
    account_id: Optional[str]
    batch_id: Optional[str]
    request_id: Optional[str]
    type: Optional[MarketIfTouchedOrderRejectTransactionType]
    instrument: Optional[str]
    units: Optional[str]
    price: Optional[str]
    price_bound: Optional[str]
    time_in_force: Optional[MarketIfTouchedOrderRejectTransactionTimeInForce]
    gtd_time: Optional[str]
    position_fill: Optional[MarketIfTouchedOrderRejectTransactionPositionFill]
    trigger_condition: Optional[MarketIfTouchedOrderRejectTransactionTriggerCondition]
    reason: Optional[MarketIfTouchedOrderRejectTransactionReason]
    client_extensions: Optional["ClientExtensions"]
    take_profit_on_fill: Optional["TakeProfitDetails"]
    stop_loss_on_fill: Optional["StopLossDetails"]
    trailing_stop_loss_on_fill: Optional["TrailingStopLossDetails"]
    trade_client_extensions: Optional["ClientExtensions"]
    intended_replaces_order_id: Optional[str]
    reject_reason: Optional[MarketIfTouchedOrderRejectTransactionRejectReason]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .take_profit_details import TakeProfitDetails
        from .trailing_stop_loss_details import TrailingStopLossDetails
        from .client_extensions import ClientExtensions
        from .stop_loss_details import StopLossDetails

        d = src_dict.copy()
        id = d.pop("id", None)
        time = d.pop("time", None)
        user_id = d.pop("userID", None)
        account_id = d.pop("accountID", None)
        batch_id = d.pop("batchID", None)
        request_id = d.pop("requestID", None)
        _type = d.pop("type", None)
        type: Optional[MarketIfTouchedOrderRejectTransactionType]
        if _type is None:
            type = None
        else:
            type = check_market_if_touched_order_reject_transaction_type(_type)
        instrument = d.pop("instrument", None)
        units = d.pop("units", None)
        price = d.pop("price", None)
        price_bound = d.pop("priceBound", None)
        _time_in_force = d.pop("timeInForce", None)
        time_in_force: Optional[MarketIfTouchedOrderRejectTransactionTimeInForce]
        if _time_in_force is None:
            time_in_force = None
        else:
            time_in_force = (
                check_market_if_touched_order_reject_transaction_time_in_force(
                    _time_in_force
                )
            )
        gtd_time = d.pop("gtdTime", None)
        _position_fill = d.pop("positionFill", None)
        position_fill: Optional[MarketIfTouchedOrderRejectTransactionPositionFill]
        if _position_fill is None:
            position_fill = None
        else:
            position_fill = (
                check_market_if_touched_order_reject_transaction_position_fill(
                    _position_fill
                )
            )
        _trigger_condition = d.pop("triggerCondition", None)
        trigger_condition: Optional[
            MarketIfTouchedOrderRejectTransactionTriggerCondition
        ]
        if _trigger_condition is None:
            trigger_condition = None
        else:
            trigger_condition = (
                check_market_if_touched_order_reject_transaction_trigger_condition(
                    _trigger_condition
                )
            )
        _reason = d.pop("reason", None)
        reason: Optional[MarketIfTouchedOrderRejectTransactionReason]
        if _reason is None:
            reason = None
        else:
            reason = check_market_if_touched_order_reject_transaction_reason(_reason)
        _client_extensions = d.pop("clientExtensions", None)
        client_extensions: Optional[ClientExtensions]
        if _client_extensions is None:
            client_extensions = None
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)
        _take_profit_on_fill = d.pop("takeProfitOnFill", None)
        take_profit_on_fill: Optional[TakeProfitDetails]
        if _take_profit_on_fill is None:
            take_profit_on_fill = None
        else:
            take_profit_on_fill = TakeProfitDetails.from_dict(_take_profit_on_fill)
        _stop_loss_on_fill = d.pop("stopLossOnFill", None)
        stop_loss_on_fill: Optional[StopLossDetails]
        if _stop_loss_on_fill is None:
            stop_loss_on_fill = None
        else:
            stop_loss_on_fill = StopLossDetails.from_dict(_stop_loss_on_fill)
        _trailing_stop_loss_on_fill = d.pop("trailingStopLossOnFill", None)
        trailing_stop_loss_on_fill: Optional[TrailingStopLossDetails]
        if _trailing_stop_loss_on_fill is None:
            trailing_stop_loss_on_fill = None
        else:
            trailing_stop_loss_on_fill = TrailingStopLossDetails.from_dict(
                _trailing_stop_loss_on_fill
            )
        _trade_client_extensions = d.pop("tradeClientExtensions", None)
        trade_client_extensions: Optional[ClientExtensions]
        if _trade_client_extensions is None:
            trade_client_extensions = None
        else:
            trade_client_extensions = ClientExtensions.from_dict(
                _trade_client_extensions
            )
        intended_replaces_order_id = d.pop("intendedReplacesOrderID", None)
        _reject_reason = d.pop("rejectReason", None)
        reject_reason: Optional[MarketIfTouchedOrderRejectTransactionRejectReason]
        if _reject_reason is None:
            reject_reason = None
        else:
            reject_reason = (
                check_market_if_touched_order_reject_transaction_reject_reason(
                    _reject_reason
                )
            )
        market_if_touched_order_reject_transaction = cls(
            id=id,
            time=time,
            user_id=user_id,
            account_id=account_id,
            batch_id=batch_id,
            request_id=request_id,
            type=type,
            instrument=instrument,
            units=units,
            price=price,
            price_bound=price_bound,
            time_in_force=time_in_force,
            gtd_time=gtd_time,
            position_fill=position_fill,
            trigger_condition=trigger_condition,
            reason=reason,
            client_extensions=client_extensions,
            take_profit_on_fill=take_profit_on_fill,
            stop_loss_on_fill=stop_loss_on_fill,
            trailing_stop_loss_on_fill=trailing_stop_loss_on_fill,
            trade_client_extensions=trade_client_extensions,
            intended_replaces_order_id=intended_replaces_order_id,
            reject_reason=reject_reason,
        )
        market_if_touched_order_reject_transaction.additional_properties = d
        return market_if_touched_order_reject_transaction

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="OrderIdentifier")


@dataclasses.dataclass
class OrderIdentifier:
    """An OrderIdentifier is used to refer to an Order, and contains both the OrderID and the ClientOrderID.

    Attributes:
        order_id (Optional[str]): The OANDA-assigned Order ID
        client_order_id (Optional[str]): The client-provided client Order ID"""

    order_id: Optional[str]
    client_order_id: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        order_id = d.pop("orderID", None)
        client_order_id = d.pop("clientOrderID", None)
        order_identifier = cls(order_id=order_id, client_order_id=client_order_id)
        order_identifier.additional_properties = d
        return order_identifier

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from http import HTTPStatus
from typing import Any, Dict, Optional, Union, cast

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response
from ... import errors

from ...models.list_positions_response_200 import ListPositionsResponse200


def _get_kwargs(
    account_id: str,
    *,
    authorization: str,
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}
    headers["Authorization"] = authorization

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/accounts/{account_id}/positions".format(
            account_id=account_id,
        ),
    }

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Any, ListPositionsResponse200]]:
    if response.status_code == 200:
        response_200 = ListPositionsResponse200.from_dict(response.json())

        return response_200
    if response.status_code == 401:
        response_401 = cast(Any, None)
        return response_401
    if response.status_code == 404:
        response_404 = cast(Any, None)
        return response_404
    if response.status_code == 405:
        response_405 = cast(Any, None)
        return response_405
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, ListPositionsResponse200]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    account_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    authorization: str,
) -> Response[Union[Any, ListPositionsResponse200]]:
    """List Positions

     List all Positions for an Account. The Positions returned are for every instrument that has had a
    position during the lifetime of an the Account.

    Args:
        account_id (str):
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, ListPositionsResponse200]]
    """

    kwargs = _get_kwargs(
        account_id=account_id,
        authorization=authorization,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    account_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    authorization: str,
) -> Optional[Union[Any, ListPositionsResponse200]]:
    """List Positions

     List all Positions for an Account. The Positions returned are for every instrument that has had a
    position during the lifetime of an the Account.

    Args:
        account_id (str):
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, ListPositionsResponse200]
    """

    return sync_detailed(
        account_id=account_id,
        client=client,
        authorization=authorization,
    ).parsed


async def asyncio_detailed(
    account_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    authorization: str,
) -> Response[Union[Any, ListPositionsResponse200]]:
    """List Positions

     List all Positions for an Account. The Positions returned are for every instrument that has had a
    position during the lifetime of an the Account.

    Args:
        account_id (str):
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, ListPositionsResponse200]]
    """

    kwargs = _get_kwargs(
        account_id=account_id,
        authorization=authorization,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    account_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    authorization: str,
) -> Optional[Union[Any, ListPositionsResponse200]]:
    """List Positions

     List all Positions for an Account. The Positions returned are for every instrument that has had a
    position during the lifetime of an the Account.

    Args:
        account_id (str):
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, ListPositionsResponse200]
    """

    return (
        await asyncio_detailed(
            account_id=account_id,
            client=client,
            authorization=authorization,
        )
    ).parsed



from __future__ import annotations
from typing import Literal, Set, cast

DelayedTradeClosureTransactionReason = Literal[
    "CLIENT_ORDER",
    "DELAYED_TRADE_CLOSE",
    "MARGIN_CLOSEOUT",
    "POSITION_CLOSEOUT",
    "TRADE_CLOSE",
]
DELAYED_TRADE_CLOSURE_TRANSACTION_REASON_VALUES: Set[
    DelayedTradeClosureTransactionReason
] = {
    "CLIENT_ORDER",
    "DELAYED_TRADE_CLOSE",
    "MARGIN_CLOSEOUT",
    "POSITION_CLOSEOUT",
    "TRADE_CLOSE",
}


def check_delayed_trade_closure_transaction_reason(
    value: str,
) -> DelayedTradeClosureTransactionReason:
    if value in DELAYED_TRADE_CLOSURE_TRANSACTION_REASON_VALUES:
        return cast(DelayedTradeClosureTransactionReason, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {DELAYED_TRADE_CLOSURE_TRANSACTION_REASON_VALUES!r}"
    )



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union


T = TypeVar("T", bound="Transaction")


@_attrs_define
class Transaction:
    """The base Transaction specification. Specifies properties that are common between all Transaction.

    Attributes:
        id (Union[Unset, str]): The Transaction's Identifier.
        time (Union[Unset, str]): The date/time when the Transaction was created.
        user_id (Union[Unset, int]): The ID of the user that initiated the creation of the Transaction.
        account_id (Union[Unset, str]): The ID of the Account the Transaction was created for.
        batch_id (Union[Unset, str]): The ID of the "batch" that the Transaction belongs to. Transactions in the same
            batch are applied to the Account simultaneously.
        request_id (Union[Unset, str]): The Request ID of the request which generated the transaction.
    """

    id: Union[Unset, str] = UNSET
    time: Union[Unset, str] = UNSET
    user_id: Union[Unset, int] = UNSET
    account_id: Union[Unset, str] = UNSET
    batch_id: Union[Unset, str] = UNSET
    request_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        time = self.time

        user_id = self.user_id

        account_id = self.account_id

        batch_id = self.batch_id

        request_id = self.request_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if time is not UNSET:
            field_dict["time"] = time
        if user_id is not UNSET:
            field_dict["userID"] = user_id
        if account_id is not UNSET:
            field_dict["accountID"] = account_id
        if batch_id is not UNSET:
            field_dict["batchID"] = batch_id
        if request_id is not UNSET:
            field_dict["requestID"] = request_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        time = d.pop("time", UNSET)

        user_id = d.pop("userID", UNSET)

        account_id = d.pop("accountID", UNSET)

        batch_id = d.pop("batchID", UNSET)

        request_id = d.pop("requestID", UNSET)

        transaction = cls(
            id=id,
            time=time,
            user_id=user_id,
            account_id=account_id,
            batch_id=batch_id,
            request_id=request_id,
        )

        transaction.additional_properties = d
        return transaction

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .transaction import Transaction
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="GetTransactionResponse200")


@dataclasses.dataclass
class GetTransactionResponse200:
    """Attributes:
    transaction (Optional[Transaction]): The base Transaction specification. Specifies properties that are
        common between all Transaction.
    last_transaction_id (Optional[str]): The ID of the most recent Transaction created for the Account"""

    transaction: Optional["Transaction"]
    last_transaction_id: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .transaction import Transaction

        d = src_dict.copy()
        _transaction = d.pop("transaction", None)
        transaction: Optional[Transaction]
        if _transaction is None:
            transaction = None
        else:
            transaction = Transaction.from_dict(_transaction)
        last_transaction_id = d.pop("lastTransactionID", None)
        get_transaction_response_200 = cls(
            transaction=transaction, last_transaction_id=last_transaction_id
        )
        get_transaction_response_200.additional_properties = d
        return get_transaction_response_200

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Literal, Set, cast

TrailingStopLossOrderState = Literal["CANCELLED", "FILLED", "PENDING", "TRIGGERED"]
TRAILING_STOP_LOSS_ORDER_STATE_VALUES: Set[TrailingStopLossOrderState] = {
    "CANCELLED",
    "FILLED",
    "PENDING",
    "TRIGGERED",
}


def check_trailing_stop_loss_order_state(value: str) -> TrailingStopLossOrderState:
    if value in TRAILING_STOP_LOSS_ORDER_STATE_VALUES:
        return cast(TrailingStopLossOrderState, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {TRAILING_STOP_LOSS_ORDER_STATE_VALUES!r}"
    )



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .transaction import Transaction
from .transaction_heartbeat import TransactionHeartbeat
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="StreamTransactionsResponse200")


@dataclasses.dataclass
class StreamTransactionsResponse200:
    """The response body for the Transaction Stream uses chunked transfer encoding.  Each chunk contains Transaction and/or
    TransactionHeartbeat objects encoded as JSON.  Each JSON object is serialized into a single line of text, and
    multiple objects found in the same chunk are separated by newlines.
    TransactionHeartbeats are sent every 5 seconds.

        Attributes:
            transaction (Optional[Transaction]): The base Transaction specification. Specifies properties that are
                common between all Transaction.
            heartbeat (Optional[TransactionHeartbeat]): A TransactionHeartbeat object is injected into the Transaction
                stream to ensure that the HTTP connection remains active."""

    transaction: Optional["Transaction"]
    heartbeat: Optional["TransactionHeartbeat"]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .transaction import Transaction
        from .transaction_heartbeat import TransactionHeartbeat

        d = src_dict.copy()
        _transaction = d.pop("transaction", None)
        transaction: Optional[Transaction]
        if _transaction is None:
            transaction = None
        else:
            transaction = Transaction.from_dict(_transaction)
        _heartbeat = d.pop("heartbeat", None)
        heartbeat: Optional[TransactionHeartbeat]
        if _heartbeat is None:
            heartbeat = None
        else:
            heartbeat = TransactionHeartbeat.from_dict(_heartbeat)
        stream_transactions_response_200 = cls(
            transaction=transaction, heartbeat=heartbeat
        )
        stream_transactions_response_200.additional_properties = d
        return stream_transactions_response_200

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .client_price import ClientPrice
from .pricing_heartbeat import PricingHeartbeat
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="StreamPricingResponse200")


@dataclasses.dataclass
class StreamPricingResponse200:
    """The response body for the Pricing Stream uses chunked transfer encoding.  Each chunk contains Price and/or
    PricingHeartbeat objects encoded as JSON.  Each JSON object is serialized into a single line of text, and multiple
    objects found in the same chunk are separated by newlines.
    Heartbeats are sent every 5 seconds.

        Attributes:
            price (Optional[ClientPrice]): The specification of an Account-specific Price.
            heartbeat (Optional[PricingHeartbeat]): A PricingHeartbeat object is injected into the Pricing stream to
                ensure that the HTTP connection remains active."""

    price: Optional["ClientPrice"]
    heartbeat: Optional["PricingHeartbeat"]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .client_price import ClientPrice
        from .pricing_heartbeat import PricingHeartbeat

        d = src_dict.copy()
        _price = d.pop("price", None)
        price: Optional[ClientPrice]
        if _price is None:
            price = None
        else:
            price = ClientPrice.from_dict(_price)
        _heartbeat = d.pop("heartbeat", None)
        heartbeat: Optional[PricingHeartbeat]
        if _heartbeat is None:
            heartbeat = None
        else:
            heartbeat = PricingHeartbeat.from_dict(_heartbeat)
        stream_pricing_response_200 = cls(price=price, heartbeat=heartbeat)
        stream_pricing_response_200.additional_properties = d
        return stream_pricing_response_200

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from typing import List, Optional, Type, TypeVar, cast

T = TypeVar("T", bound="AccountProperties")


@dataclasses.dataclass
class AccountProperties:
    """Properties related to an Account.

    Attributes:
        id (Optional[str]): The Account's identifier
        mt_4_account_id (Optional[int]): The Account's associated MT4 Account ID. This field will not be present if
            the Account is not an MT4 account.
        tags (Optional[List[str]]): The Account's tags"""

    id: Optional[str]
    mt_4_account_id: Optional[int]
    tags: Optional[List[str]]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", None)
        mt_4_account_id = d.pop("mt4AccountID", None)
        tags = cast(List[str], d.pop("tags", None))
        account_properties = cls(id=id, mt_4_account_id=mt_4_account_id, tags=tags)
        account_properties.additional_properties = d
        return account_properties

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast
from typing import Union


T = TypeVar("T", bound="PositionSide")


@_attrs_define
class PositionSide:
    """The representation of a Position for a single direction (long or short).

    Attributes:
        units (Union[Unset, str]): Number of units in the position (negative value indicates short position, positive
            indicates long position).
        average_price (Union[Unset, str]): Volume-weighted average of the underlying Trade open prices for the Position.
        trade_i_ds (Union[Unset, List[str]]): List of the open Trade IDs which contribute to the open Position.
        pl (Union[Unset, str]): Profit/loss realized by the PositionSide over the lifetime of the Account.
        unrealized_pl (Union[Unset, str]): The unrealized profit/loss of all open Trades that contribute to this
            PositionSide.
        resettable_pl (Union[Unset, str]): Profit/loss realized by the PositionSide since the Account's resettablePL was
            last reset by the client.
        financing (Union[Unset, str]): The total amount of financing paid/collected for this PositionSide over the
            lifetime of the Account.
        guaranteed_execution_fees (Union[Unset, str]): The total amount of fees charged over the lifetime of the Account
            for the execution of guaranteed Stop Loss Orders attached to Trades for this PositionSide.
    """

    units: Union[Unset, str] = UNSET
    average_price: Union[Unset, str] = UNSET
    trade_i_ds: Union[Unset, List[str]] = UNSET
    pl: Union[Unset, str] = UNSET
    unrealized_pl: Union[Unset, str] = UNSET
    resettable_pl: Union[Unset, str] = UNSET
    financing: Union[Unset, str] = UNSET
    guaranteed_execution_fees: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        units = self.units

        average_price = self.average_price

        trade_i_ds: Union[Unset, List[str]] = UNSET
        if not isinstance(self.trade_i_ds, Unset):
            trade_i_ds = self.trade_i_ds

        pl = self.pl

        unrealized_pl = self.unrealized_pl

        resettable_pl = self.resettable_pl

        financing = self.financing

        guaranteed_execution_fees = self.guaranteed_execution_fees

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if units is not UNSET:
            field_dict["units"] = units
        if average_price is not UNSET:
            field_dict["averagePrice"] = average_price
        if trade_i_ds is not UNSET:
            field_dict["tradeIDs"] = trade_i_ds
        if pl is not UNSET:
            field_dict["pl"] = pl
        if unrealized_pl is not UNSET:
            field_dict["unrealizedPL"] = unrealized_pl
        if resettable_pl is not UNSET:
            field_dict["resettablePL"] = resettable_pl
        if financing is not UNSET:
            field_dict["financing"] = financing
        if guaranteed_execution_fees is not UNSET:
            field_dict["guaranteedExecutionFees"] = guaranteed_execution_fees

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        units = d.pop("units", UNSET)

        average_price = d.pop("averagePrice", UNSET)

        trade_i_ds = cast(List[str], d.pop("tradeIDs", UNSET))

        pl = d.pop("pl", UNSET)

        unrealized_pl = d.pop("unrealizedPL", UNSET)

        resettable_pl = d.pop("resettablePL", UNSET)

        financing = d.pop("financing", UNSET)

        guaranteed_execution_fees = d.pop("guaranteedExecutionFees", UNSET)

        position_side = cls(
            units=units,
            average_price=average_price,
            trade_i_ds=trade_i_ds,
            pl=pl,
            unrealized_pl=unrealized_pl,
            resettable_pl=resettable_pl,
            financing=financing,
            guaranteed_execution_fees=guaranteed_execution_fees,
        )

        position_side.additional_properties = d
        return position_side

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations
from typing import Literal, Set, cast

TrailingStopLossOrderRequestTriggerCondition = Literal[
    "ASK", "BID", "DEFAULT", "INVERSE", "MID"
]
TRAILING_STOP_LOSS_ORDER_REQUEST_TRIGGER_CONDITION_VALUES: Set[
    TrailingStopLossOrderRequestTriggerCondition
] = {"ASK", "BID", "DEFAULT", "INVERSE", "MID"}


def check_trailing_stop_loss_order_request_trigger_condition(
    value: str,
) -> TrailingStopLossOrderRequestTriggerCondition:
    if value in TRAILING_STOP_LOSS_ORDER_REQUEST_TRIGGER_CONDITION_VALUES:
        return cast(TrailingStopLossOrderRequestTriggerCondition, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {TRAILING_STOP_LOSS_ORDER_REQUEST_TRIGGER_CONDITION_VALUES!r}"
    )



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast
from typing import Union

if TYPE_CHECKING:
    from ..models.order_cancel_reject_transaction import OrderCancelRejectTransaction
    from ..models.take_profit_order_reject_transaction import (
        TakeProfitOrderRejectTransaction,
    )
    from ..models.stop_loss_order_reject_transaction import (
        StopLossOrderRejectTransaction,
    )
    from ..models.trailing_stop_loss_order_reject_transaction import (
        TrailingStopLossOrderRejectTransaction,
    )


T = TypeVar("T", bound="SetTradeDependentOrdersResponse400")


@_attrs_define
class SetTradeDependentOrdersResponse400:
    """
    Attributes:
        take_profit_order_cancel_reject_transaction (Union[Unset, OrderCancelRejectTransaction]): An
            OrderCancelRejectTransaction represents the rejection of the cancellation of an Order in the client's Account.
        take_profit_order_reject_transaction (Union[Unset, TakeProfitOrderRejectTransaction]): A
            TakeProfitOrderRejectTransaction represents the rejection of the creation of a TakeProfit Order.
        stop_loss_order_cancel_reject_transaction (Union[Unset, OrderCancelRejectTransaction]): An
            OrderCancelRejectTransaction represents the rejection of the cancellation of an Order in the client's Account.
        stop_loss_order_reject_transaction (Union[Unset, StopLossOrderRejectTransaction]): A
            StopLossOrderRejectTransaction represents the rejection of the creation of a StopLoss Order.
        trailing_stop_loss_order_cancel_reject_transaction (Union[Unset, OrderCancelRejectTransaction]): An
            OrderCancelRejectTransaction represents the rejection of the cancellation of an Order in the client's Account.
        trailing_stop_loss_order_reject_transaction (Union[Unset, TrailingStopLossOrderRejectTransaction]): A
            TrailingStopLossOrderRejectTransaction represents the rejection of the creation of a TrailingStopLoss Order.
        last_transaction_id (Union[Unset, str]): The ID of the most recent Transaction created for the Account.
        related_transaction_i_ds (Union[Unset, List[str]]): The IDs of all Transactions that were created while
            satisfying the request.
        error_code (Union[Unset, str]): The code of the error that has occurred. This field may not be returned for some
            errors.
        error_message (Union[Unset, str]): The human-readable description of the error that has occurred.
    """

    take_profit_order_cancel_reject_transaction: Union[
        Unset, "OrderCancelRejectTransaction"
    ] = UNSET
    take_profit_order_reject_transaction: Union[
        Unset, "TakeProfitOrderRejectTransaction"
    ] = UNSET
    stop_loss_order_cancel_reject_transaction: Union[
        Unset, "OrderCancelRejectTransaction"
    ] = UNSET
    stop_loss_order_reject_transaction: Union[
        Unset, "StopLossOrderRejectTransaction"
    ] = UNSET
    trailing_stop_loss_order_cancel_reject_transaction: Union[
        Unset, "OrderCancelRejectTransaction"
    ] = UNSET
    trailing_stop_loss_order_reject_transaction: Union[
        Unset, "TrailingStopLossOrderRejectTransaction"
    ] = UNSET
    last_transaction_id: Union[Unset, str] = UNSET
    related_transaction_i_ds: Union[Unset, List[str]] = UNSET
    error_code: Union[Unset, str] = UNSET
    error_message: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        take_profit_order_cancel_reject_transaction: Union[Unset, Dict[str, Any]] = (
            UNSET
        )
        if not isinstance(self.take_profit_order_cancel_reject_transaction, Unset):
            take_profit_order_cancel_reject_transaction = (
                self.take_profit_order_cancel_reject_transaction.to_dict()
            )

        take_profit_order_reject_transaction: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.take_profit_order_reject_transaction, Unset):
            take_profit_order_reject_transaction = (
                self.take_profit_order_reject_transaction.to_dict()
            )

        stop_loss_order_cancel_reject_transaction: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.stop_loss_order_cancel_reject_transaction, Unset):
            stop_loss_order_cancel_reject_transaction = (
                self.stop_loss_order_cancel_reject_transaction.to_dict()
            )

        stop_loss_order_reject_transaction: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.stop_loss_order_reject_transaction, Unset):
            stop_loss_order_reject_transaction = (
                self.stop_loss_order_reject_transaction.to_dict()
            )

        trailing_stop_loss_order_cancel_reject_transaction: Union[
            Unset, Dict[str, Any]
        ] = UNSET
        if not isinstance(
            self.trailing_stop_loss_order_cancel_reject_transaction, Unset
        ):
            trailing_stop_loss_order_cancel_reject_transaction = (
                self.trailing_stop_loss_order_cancel_reject_transaction.to_dict()
            )

        trailing_stop_loss_order_reject_transaction: Union[Unset, Dict[str, Any]] = (
            UNSET
        )
        if not isinstance(self.trailing_stop_loss_order_reject_transaction, Unset):
            trailing_stop_loss_order_reject_transaction = (
                self.trailing_stop_loss_order_reject_transaction.to_dict()
            )

        last_transaction_id = self.last_transaction_id

        related_transaction_i_ds: Union[Unset, List[str]] = UNSET
        if not isinstance(self.related_transaction_i_ds, Unset):
            related_transaction_i_ds = self.related_transaction_i_ds

        error_code = self.error_code

        error_message = self.error_message

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if take_profit_order_cancel_reject_transaction is not UNSET:
            field_dict["takeProfitOrderCancelRejectTransaction"] = (
                take_profit_order_cancel_reject_transaction
            )
        if take_profit_order_reject_transaction is not UNSET:
            field_dict["takeProfitOrderRejectTransaction"] = (
                take_profit_order_reject_transaction
            )
        if stop_loss_order_cancel_reject_transaction is not UNSET:
            field_dict["stopLossOrderCancelRejectTransaction"] = (
                stop_loss_order_cancel_reject_transaction
            )
        if stop_loss_order_reject_transaction is not UNSET:
            field_dict["stopLossOrderRejectTransaction"] = (
                stop_loss_order_reject_transaction
            )
        if trailing_stop_loss_order_cancel_reject_transaction is not UNSET:
            field_dict["trailingStopLossOrderCancelRejectTransaction"] = (
                trailing_stop_loss_order_cancel_reject_transaction
            )
        if trailing_stop_loss_order_reject_transaction is not UNSET:
            field_dict["trailingStopLossOrderRejectTransaction"] = (
                trailing_stop_loss_order_reject_transaction
            )
        if last_transaction_id is not UNSET:
            field_dict["lastTransactionID"] = last_transaction_id
        if related_transaction_i_ds is not UNSET:
            field_dict["relatedTransactionIDs"] = related_transaction_i_ds
        if error_code is not UNSET:
            field_dict["errorCode"] = error_code
        if error_message is not UNSET:
            field_dict["errorMessage"] = error_message

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.order_cancel_reject_transaction import (
            OrderCancelRejectTransaction,
        )
        from ..models.take_profit_order_reject_transaction import (
            TakeProfitOrderRejectTransaction,
        )
        from ..models.stop_loss_order_reject_transaction import (
            StopLossOrderRejectTransaction,
        )
        from ..models.trailing_stop_loss_order_reject_transaction import (
            TrailingStopLossOrderRejectTransaction,
        )

        d = src_dict.copy()
        _take_profit_order_cancel_reject_transaction = d.pop(
            "takeProfitOrderCancelRejectTransaction", UNSET
        )
        take_profit_order_cancel_reject_transaction: Union[
            Unset, OrderCancelRejectTransaction
        ]
        if isinstance(_take_profit_order_cancel_reject_transaction, Unset):
            take_profit_order_cancel_reject_transaction = UNSET
        else:
            take_profit_order_cancel_reject_transaction = (
                OrderCancelRejectTransaction.from_dict(
                    _take_profit_order_cancel_reject_transaction
                )
            )

        _take_profit_order_reject_transaction = d.pop(
            "takeProfitOrderRejectTransaction", UNSET
        )
        take_profit_order_reject_transaction: Union[
            Unset, TakeProfitOrderRejectTransaction
        ]
        if isinstance(_take_profit_order_reject_transaction, Unset):
            take_profit_order_reject_transaction = UNSET
        else:
            take_profit_order_reject_transaction = (
                TakeProfitOrderRejectTransaction.from_dict(
                    _take_profit_order_reject_transaction
                )
            )

        _stop_loss_order_cancel_reject_transaction = d.pop(
            "stopLossOrderCancelRejectTransaction", UNSET
        )
        stop_loss_order_cancel_reject_transaction: Union[
            Unset, OrderCancelRejectTransaction
        ]
        if isinstance(_stop_loss_order_cancel_reject_transaction, Unset):
            stop_loss_order_cancel_reject_transaction = UNSET
        else:
            stop_loss_order_cancel_reject_transaction = (
                OrderCancelRejectTransaction.from_dict(
                    _stop_loss_order_cancel_reject_transaction
                )
            )

        _stop_loss_order_reject_transaction = d.pop(
            "stopLossOrderRejectTransaction", UNSET
        )
        stop_loss_order_reject_transaction: Union[Unset, StopLossOrderRejectTransaction]
        if isinstance(_stop_loss_order_reject_transaction, Unset):
            stop_loss_order_reject_transaction = UNSET
        else:
            stop_loss_order_reject_transaction = (
                StopLossOrderRejectTransaction.from_dict(
                    _stop_loss_order_reject_transaction
                )
            )

        _trailing_stop_loss_order_cancel_reject_transaction = d.pop(
            "trailingStopLossOrderCancelRejectTransaction", UNSET
        )
        trailing_stop_loss_order_cancel_reject_transaction: Union[
            Unset, OrderCancelRejectTransaction
        ]
        if isinstance(_trailing_stop_loss_order_cancel_reject_transaction, Unset):
            trailing_stop_loss_order_cancel_reject_transaction = UNSET
        else:
            trailing_stop_loss_order_cancel_reject_transaction = (
                OrderCancelRejectTransaction.from_dict(
                    _trailing_stop_loss_order_cancel_reject_transaction
                )
            )

        _trailing_stop_loss_order_reject_transaction = d.pop(
            "trailingStopLossOrderRejectTransaction", UNSET
        )
        trailing_stop_loss_order_reject_transaction: Union[
            Unset, TrailingStopLossOrderRejectTransaction
        ]
        if isinstance(_trailing_stop_loss_order_reject_transaction, Unset):
            trailing_stop_loss_order_reject_transaction = UNSET
        else:
            trailing_stop_loss_order_reject_transaction = (
                TrailingStopLossOrderRejectTransaction.from_dict(
                    _trailing_stop_loss_order_reject_transaction
                )
            )

        last_transaction_id = d.pop("lastTransactionID", UNSET)

        related_transaction_i_ds = cast(
            List[str], d.pop("relatedTransactionIDs", UNSET)
        )

        error_code = d.pop("errorCode", UNSET)

        error_message = d.pop("errorMessage", UNSET)

        set_trade_dependent_orders_response_400 = cls(
            take_profit_order_cancel_reject_transaction=take_profit_order_cancel_reject_transaction,
            take_profit_order_reject_transaction=take_profit_order_reject_transaction,
            stop_loss_order_cancel_reject_transaction=stop_loss_order_cancel_reject_transaction,
            stop_loss_order_reject_transaction=stop_loss_order_reject_transaction,
            trailing_stop_loss_order_cancel_reject_transaction=trailing_stop_loss_order_cancel_reject_transaction,
            trailing_stop_loss_order_reject_transaction=trailing_stop_loss_order_reject_transaction,
            last_transaction_id=last_transaction_id,
            related_transaction_i_ds=related_transaction_i_ds,
            error_code=error_code,
            error_message=error_message,
        )

        set_trade_dependent_orders_response_400.additional_properties = d
        return set_trade_dependent_orders_response_400

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union


T = TypeVar("T", bound="PricingHeartbeat")


@_attrs_define
class PricingHeartbeat:
    """A PricingHeartbeat object is injected into the Pricing stream to ensure that the HTTP connection remains active.

    Attributes:
        type (Union[Unset, str]): The string "HEARTBEAT"
        time (Union[Unset, str]): The date/time when the Heartbeat was created.
    """

    type: Union[Unset, str] = UNSET
    time: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type

        time = self.time

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type
        if time is not UNSET:
            field_dict["time"] = time

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        type = d.pop("type", UNSET)

        time = d.pop("time", UNSET)

        pricing_heartbeat = cls(
            type=type,
            time=time,
        )

        pricing_heartbeat.additional_properties = d
        return pricing_heartbeat

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Literal, Set, cast

InstrumentType = Literal["CFD", "CURRENCY", "METAL"]

INSTRUMENT_TYPE_VALUES: Set[InstrumentType] = {
    "CFD",
    "CURRENCY",
    "METAL",
}


def check_instrument_type(value: str) -> InstrumentType:
    if value in INSTRUMENT_TYPE_VALUES:
        return cast(InstrumentType, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {INSTRUMENT_TYPE_VALUES!r}"
    )



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .client_extensions import ClientExtensions
from .trailing_stop_loss_details_time_in_force import TrailingStopLossDetailsTimeInForce
from .trailing_stop_loss_details_time_in_force import (
    check_trailing_stop_loss_details_time_in_force,
)
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="TrailingStopLossDetails")


@dataclasses.dataclass
class TrailingStopLossDetails:
    """TrailingStopLossDetails specifies the details of a Trailing Stop Loss Order to be created on behalf of a client.
    This may happen when an Order is filled that opens a Trade requiring a Trailing Stop Loss, or when a Trade's
    dependent Trailing Stop Loss Order is modified directly through the Trade.

        Attributes:
            distance (Optional[str]): The distance (in price units) from the Trade's fill price that the Trailing Stop
                Loss Order will be triggered at.
            time_in_force (Optional[TrailingStopLossDetailsTimeInForce]): The time in force for the created Trailing
                Stop Loss Order. This may only be GTC, GTD or GFD.
            gtd_time (Optional[str]): The date when the Trailing Stop Loss Order will be cancelled on if timeInForce is
                GTD.
            client_extensions (Optional[ClientExtensions]): A ClientExtensions object allows a client to attach a
                clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
                your account is associated with MT4."""

    distance: Optional[str]
    time_in_force: Optional[TrailingStopLossDetailsTimeInForce]
    gtd_time: Optional[str]
    client_extensions: Optional["ClientExtensions"]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .client_extensions import ClientExtensions

        d = src_dict.copy()
        distance = d.pop("distance", None)
        _time_in_force = d.pop("timeInForce", None)
        time_in_force: Optional[TrailingStopLossDetailsTimeInForce]
        if _time_in_force is None:
            time_in_force = None
        else:
            time_in_force = check_trailing_stop_loss_details_time_in_force(
                _time_in_force
            )
        gtd_time = d.pop("gtdTime", None)
        _client_extensions = d.pop("clientExtensions", None)
        client_extensions: Optional[ClientExtensions]
        if _client_extensions is None:
            client_extensions = None
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)
        trailing_stop_loss_details = cls(
            distance=distance,
            time_in_force=time_in_force,
            gtd_time=gtd_time,
            client_extensions=client_extensions,
        )
        trailing_stop_loss_details.additional_properties = d
        return trailing_stop_loss_details

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union

if TYPE_CHECKING:
    from ..models.units_available_details import UnitsAvailableDetails


T = TypeVar("T", bound="UnitsAvailable")


@_attrs_define
class UnitsAvailable:
    """Representation of how many units of an Instrument are available to be traded by an Order depending on its
    postionFill option.

        Attributes:
            default (Union[Unset, UnitsAvailableDetails]): Representation of many units of an Instrument are available to be
                traded for both long and short Orders.
            reduce_first (Union[Unset, UnitsAvailableDetails]): Representation of many units of an Instrument are available
                to be traded for both long and short Orders.
            reduce_only (Union[Unset, UnitsAvailableDetails]): Representation of many units of an Instrument are available
                to be traded for both long and short Orders.
            open_only (Union[Unset, UnitsAvailableDetails]): Representation of many units of an Instrument are available to
                be traded for both long and short Orders.
    """

    default: Union[Unset, "UnitsAvailableDetails"] = UNSET
    reduce_first: Union[Unset, "UnitsAvailableDetails"] = UNSET
    reduce_only: Union[Unset, "UnitsAvailableDetails"] = UNSET
    open_only: Union[Unset, "UnitsAvailableDetails"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        default: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.default, Unset):
            default = self.default.to_dict()

        reduce_first: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.reduce_first, Unset):
            reduce_first = self.reduce_first.to_dict()

        reduce_only: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.reduce_only, Unset):
            reduce_only = self.reduce_only.to_dict()

        open_only: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.open_only, Unset):
            open_only = self.open_only.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if default is not UNSET:
            field_dict["default"] = default
        if reduce_first is not UNSET:
            field_dict["reduceFirst"] = reduce_first
        if reduce_only is not UNSET:
            field_dict["reduceOnly"] = reduce_only
        if open_only is not UNSET:
            field_dict["openOnly"] = open_only

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.units_available_details import UnitsAvailableDetails

        d = src_dict.copy()
        _default = d.pop("default", UNSET)
        default: Union[Unset, UnitsAvailableDetails]
        if isinstance(_default, Unset):
            default = UNSET
        else:
            default = UnitsAvailableDetails.from_dict(_default)

        _reduce_first = d.pop("reduceFirst", UNSET)
        reduce_first: Union[Unset, UnitsAvailableDetails]
        if isinstance(_reduce_first, Unset):
            reduce_first = UNSET
        else:
            reduce_first = UnitsAvailableDetails.from_dict(_reduce_first)

        _reduce_only = d.pop("reduceOnly", UNSET)
        reduce_only: Union[Unset, UnitsAvailableDetails]
        if isinstance(_reduce_only, Unset):
            reduce_only = UNSET
        else:
            reduce_only = UnitsAvailableDetails.from_dict(_reduce_only)

        _open_only = d.pop("openOnly", UNSET)
        open_only: Union[Unset, UnitsAvailableDetails]
        if isinstance(_open_only, Unset):
            open_only = UNSET
        else:
            open_only = UnitsAvailableDetails.from_dict(_open_only)

        units_available = cls(
            default=default,
            reduce_first=reduce_first,
            reduce_only=reduce_only,
            open_only=open_only,
        )

        units_available.additional_properties = d
        return units_available

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .open_trade_financing import OpenTradeFinancing
from typing import List, Optional, Type, TypeVar

T = TypeVar("T", bound="PositionFinancing")


@dataclasses.dataclass
class PositionFinancing:
    """OpenTradeFinancing is used to pay/collect daily financing charge for a Position within an Account

    Attributes:
        instrument (Optional[str]): The instrument of the Position that financing is being paid/collected for.
        financing (Optional[str]): The amount of financing paid/collected for the Position.
        open_trade_financings (Optional[List['OpenTradeFinancing']]): The financing paid/collecte for each open
            Trade within the Position."""

    instrument: Optional[str]
    financing: Optional[str]
    open_trade_financings: Optional[List["OpenTradeFinancing"]]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .open_trade_financing import OpenTradeFinancing

        d = src_dict.copy()
        instrument = d.pop("instrument", None)
        financing = d.pop("financing", None)
        open_trade_financings = []
        _open_trade_financings = d.pop("openTradeFinancings", None)
        for open_trade_financings_item_data in _open_trade_financings or []:
            open_trade_financings_item = OpenTradeFinancing.from_dict(
                open_trade_financings_item_data
            )
            open_trade_financings.append(open_trade_financings_item)
        position_financing = cls(
            instrument=instrument,
            financing=financing,
            open_trade_financings=open_trade_financings,
        )
        position_financing.additional_properties = d
        return position_financing

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .price import Price
from typing import List, Optional, Type, TypeVar

T = TypeVar("T", bound="GetPriceRangeResponse200")


@dataclasses.dataclass
class GetPriceRangeResponse200:
    """Attributes:
    prices (Optional[List['Price']]): The list of prices that satisfy the request."""

    prices: Optional[List["Price"]]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .price import Price

        d = src_dict.copy()
        prices = []
        _prices = d.pop("prices", None)
        for prices_item_data in _prices or []:
            prices_item = Price.from_dict(prices_item_data)
            prices.append(prices_item)
        get_price_range_response_200 = cls(prices=prices)
        get_price_range_response_200.additional_properties = d
        return get_price_range_response_200

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .instrument_commission import InstrumentCommission
from .instrument_type import InstrumentType
from .instrument_type import check_instrument_type
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="Instrument")


@dataclasses.dataclass
class Instrument:
    """Full specification of an Instrument.

    Attributes:
        name (Optional[str]): The name of the Instrument
        type (Optional[InstrumentType]): The type of the Instrument
        display_name (Optional[str]): The display name of the Instrument
        pip_location (Optional[int]): The location of the "pip" for this instrument. The decimal position of the pip
            in this Instrument's price can be found at 10 ^ pipLocation (e.g. -4 pipLocation results in a decimal pip
            position of 10 ^ -4 = 0.0001).
        display_precision (Optional[int]): The number of decimal places that should be used to display prices for
            this instrument. (e.g. a displayPrecision of 5 would result in a price of "1" being displayed as "1.00000")
        trade_units_precision (Optional[int]): The amount of decimal places that may be provided when specifying the
            number of units traded for this instrument.
        minimum_trade_size (Optional[str]): The smallest number of units allowed to be traded for this instrument.
        maximum_trailing_stop_distance (Optional[str]): The maximum trailing stop distance allowed for a trailing
            stop loss created for this instrument. Specified in price units.
        minimum_trailing_stop_distance (Optional[str]): The minimum trailing stop distance allowed for a trailing
            stop loss created for this instrument. Specified in price units.
        maximum_position_size (Optional[str]): The maximum position size allowed for this instrument. Specified in
            units.
        maximum_order_units (Optional[str]): The maximum units allowed for an Order placed for this instrument.
            Specified in units.
        margin_rate (Optional[str]): The margin rate for this instrument.
        commission (Optional[InstrumentCommission]): An InstrumentCommission represents an instrument-specific
            commission"""

    name: Optional[str]
    type: Optional[InstrumentType]
    display_name: Optional[str]
    pip_location: Optional[int]
    display_precision: Optional[int]
    trade_units_precision: Optional[int]
    minimum_trade_size: Optional[str]
    maximum_trailing_stop_distance: Optional[str]
    minimum_trailing_stop_distance: Optional[str]
    maximum_position_size: Optional[str]
    maximum_order_units: Optional[str]
    margin_rate: Optional[str]
    commission: Optional["InstrumentCommission"]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .instrument_commission import InstrumentCommission

        d = src_dict.copy()
        name = d.pop("name", None)
        _type = d.pop("type", None)
        type: Optional[InstrumentType]
        if _type is None:
            type = None
        else:
            type = check_instrument_type(_type)
        display_name = d.pop("displayName", None)
        pip_location = d.pop("pipLocation", None)
        display_precision = d.pop("displayPrecision", None)
        trade_units_precision = d.pop("tradeUnitsPrecision", None)
        minimum_trade_size = d.pop("minimumTradeSize", None)
        maximum_trailing_stop_distance = d.pop("maximumTrailingStopDistance", None)
        minimum_trailing_stop_distance = d.pop("minimumTrailingStopDistance", None)
        maximum_position_size = d.pop("maximumPositionSize", None)
        maximum_order_units = d.pop("maximumOrderUnits", None)
        margin_rate = d.pop("marginRate", None)
        _commission = d.pop("commission", None)
        commission: Optional[InstrumentCommission]
        if _commission is None:
            commission = None
        else:
            commission = InstrumentCommission.from_dict(_commission)
        instrument = cls(
            name=name,
            type=type,
            display_name=display_name,
            pip_location=pip_location,
            display_precision=display_precision,
            trade_units_precision=trade_units_precision,
            minimum_trade_size=minimum_trade_size,
            maximum_trailing_stop_distance=maximum_trailing_stop_distance,
            minimum_trailing_stop_distance=minimum_trailing_stop_distance,
            maximum_position_size=maximum_position_size,
            maximum_order_units=maximum_order_units,
            margin_rate=margin_rate,
            commission=commission,
        )
        instrument.additional_properties = d
        return instrument

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .client_extensions import ClientExtensions
from .take_profit_details_time_in_force import TakeProfitDetailsTimeInForce
from .take_profit_details_time_in_force import check_take_profit_details_time_in_force
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="TakeProfitDetails")


@dataclasses.dataclass
class TakeProfitDetails:
    """TakeProfitDetails specifies the details of a Take Profit Order to be created on behalf of a client. This may happen
    when an Order is filled that opens a Trade requiring a Take Profit, or when a Trade's dependent Take Profit Order is
    modified directly through the Trade.

        Attributes:
            price (Optional[str]): The price that the Take Profit Order will be triggered at. Only one of the price and
                distance fields may be specified.
            time_in_force (Optional[TakeProfitDetailsTimeInForce]): The time in force for the created Take Profit Order.
                This may only be GTC, GTD or GFD.
            gtd_time (Optional[str]): The date when the Take Profit Order will be cancelled on if timeInForce is GTD.
            client_extensions (Optional[ClientExtensions]): A ClientExtensions object allows a client to attach a
                clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
                your account is associated with MT4."""

    price: Optional[str]
    time_in_force: Optional[TakeProfitDetailsTimeInForce]
    gtd_time: Optional[str]
    client_extensions: Optional["ClientExtensions"]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .client_extensions import ClientExtensions

        d = src_dict.copy()
        price = d.pop("price", None)
        _time_in_force = d.pop("timeInForce", None)
        time_in_force: Optional[TakeProfitDetailsTimeInForce]
        if _time_in_force is None:
            time_in_force = None
        else:
            time_in_force = check_take_profit_details_time_in_force(_time_in_force)
        gtd_time = d.pop("gtdTime", None)
        _client_extensions = d.pop("clientExtensions", None)
        client_extensions: Optional[ClientExtensions]
        if _client_extensions is None:
            client_extensions = None
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)
        take_profit_details = cls(
            price=price,
            time_in_force=time_in_force,
            gtd_time=gtd_time,
            client_extensions=client_extensions,
        )
        take_profit_details.additional_properties = d
        return take_profit_details

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="UserInfo")


@dataclasses.dataclass
class UserInfo:
    """A representation of user information, as provided to the user themself.

    Attributes:
        username (Optional[str]): The user-provided username.
        user_id (Optional[int]): The user's OANDA-assigned user ID.
        country (Optional[str]): The country that the user is based in.
        email_address (Optional[str]): The user's email address."""

    username: Optional[str]
    user_id: Optional[int]
    country: Optional[str]
    email_address: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        username = d.pop("username", None)
        user_id = d.pop("userID", None)
        country = d.pop("country", None)
        email_address = d.pop("emailAddress", None)
        user_info = cls(
            username=username,
            user_id=user_id,
            country=country,
            email_address=email_address,
        )
        user_info.additional_properties = d
        return user_info

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)
